/***************************************************************************//**
 * @file        TIM/TIM_Base/main.c
 * @version     V2.0.1
 * @author      Software Development
 * @brief       Main program body.
 * @copyright   Copyright (C) Software Development. All rights reserved.
 ****************************************************************************/
 

/*****************************************************************************
 * @includes 
*****************************************************************************/

#include <stdio.h>
#include "cs32f10x_tim.h"
#include "cs32f10x_usart.h"
#include "cs32f10x_gpio.h"
#include "cs32f10x_rcu.h"
#include "cs32f10x_misc.h"


/*****************************************************************************
 * @functions
*****************************************************************************/


void usart_config(void);
void tim_config(void);
void nvic_config(void);


/**@brief       Main program.
 *
 * @param[in]   None.
 *
 * @return      None.
 */
int main(void)
{
    SCB->VTOR = FLASH_BASE | 0xA000;
    /* Configure USART3 as the printing port. */
    usart_config();
    /* Configure NVIC. */
    nvic_config();
    /* Configure TIM1 and TIM2. */
    tim_config();
    while(1);
}


/**@brief       Configure USART3 as the printing port.
 *
 * @param[in]   None.
 *
 * @return      None.
 */
void usart_config(void)
{
    usart_config_t ptr_usart;
    
    /* Enable clocks. */
    __RCU_APB2_CLK_ENABLE(RCU_APB2_PERI_GPIOB);
    __RCU_APB1_CLK_ENABLE(RCU_APB1_PERI_USART3);
    __RCU_APB2_CLK_ENABLE(RCU_APB2_PERI_AFIO);
    
    /* Configure PB10(TX) and PB11(RX). */
    gpio_mode_config(GPIOB, GPIO_PIN_10, GPIO_MODE_OUT_AFPP(GPIO_SPEED_HIGH));
    gpio_mode_config(GPIOB, GPIO_PIN_11, GPIO_MODE_IN_PU);
    
    /* Configure the basic information of USART3. */
    usart_def_init(USART3);
    ptr_usart.baud_rate = 115200;
    ptr_usart.data_width = USART_DATA_WIDTH_8;
    ptr_usart.flow_control = USART_FLOW_CONTROL_NONE;
    ptr_usart.parity_check = USART_PARITY_NONE;
    ptr_usart.stop_bits = USART_STOP_BIT_1;
    ptr_usart.transceiver_mode = USART_MODE_TX_RX;
    usart_init(USART3, &ptr_usart);
    
    /* Enable USART3. */
    __USART_ENABLE(USART3);
}


/**@brief       Configure TIM1 and TIM2.
 *
 * @param[in]   None.
 *
 * @return      None.
 */
void tim_config(void)
{
    tim_base_t ptr_time = {0};
    
    /* PCLK1 = HCLK/2 */
    rcu_pclk1_config(RCU_HCLK_DIV_2);
    /* PCLK2 = HCLK/2 */
    rcu_pclk2_config(RCU_HCLK_DIV_2);
    
    /* TIM clock enable */
    __RCU_APB2_CLK_ENABLE(RCU_APB2_PERI_TIM1);
    __RCU_APB1_CLK_ENABLE(RCU_APB1_PERI_TIM2);
    
    /* Time base configuration */
    ptr_time.clk_div = TIM_CLK_DIV1;
    ptr_time.cnt_mode = TIM_CNT_MODE_DOWN;
    ptr_time.period = 10000;
    ptr_time.pre_div = 7199;
    tim_base_init(TIM1, &ptr_time);
    ptr_time.period = 20000;
    tim_base_init(TIM2, &ptr_time);
    
    /* TIM interrupt enable */
    __TIM_INTR_ENABLE(TIM1, TIM_INTR_UPDATE);
    __TIM_INTR_ENABLE(TIM2, TIM_INTR_UPDATE);
    
    /* TIM enable */
    __TIM_ENABLE(TIM1);
    __TIM_ENABLE(TIM2);
}


/**@brief       Configure NVIC.
 *
 * @param[in]   None.
 *
 * @return      None.
 */
void nvic_config(void)
{
    nvic_init_t ptr_nvic;
    
    /* Configure and enable TIM1 interrupt. */
    ptr_nvic.nvic_irqchannel = IRQn_TIM1_UP;
    ptr_nvic.nvic_irq_enable = ENABLE;
    ptr_nvic.nvic_irq_pre_priority = 0;
    ptr_nvic.nvic_irq_sub_priority = 0;
    nvic_init(&ptr_nvic);
    
    /* Configure and enable TIM2 interrupt. */
    ptr_nvic.nvic_irqchannel = IRQn_TIM2;
    nvic_init(&ptr_nvic);
}


/**@brief       Output print information through USART3.
 *
 * @param[in]   None.
 *
 * @return      None.
 */
int fputc(int ch, FILE *f)
{
    (void) f;
    while(__USART_FLAG_STATUS_GET(USART3, TC) == RESET);
    __USART_DATA_SEND(USART3, (uint8_t) ch);

    return ch;
}

#ifdef  USE_FULL_ASSERT

/**@brief       Report the assert error.
 *
 * @param[in]   file: pointer to the source file name.
 *
 * @param[in]   line: error line source number.
 *
 * @return      None.
 */
void assert_failed(uint8_t* file, uint32_t line)
{
    while(1);
}

#endif


