
#include "csboot.h"
#include "cmd_process.h"
#include "cmsis_compiler.h"
#include "common.h"
#include "config.h"


#define OB_WRP_ADDR_START           (uint32_t)0x1FFFF808
#define OB_RDP_ADDR_START           (uint32_t)0x1FFFF800



typedef void (*Function_Pointer)(void);

extern uint8_t chip_reset_flag;
static uint32_t memory_addr;
static uint8_t memory_len;
static uint8_t cmd_execute_status;
static uint16_t ob_data[4];
static const uint8_t chip_id[4] __attribute((aligned (4))) = CHIP_ID;
static const uint8_t boot_version[4] __attribute((aligned (4))) = BOOT_VERSION;


static void csboot_cmd_get_id(void);
static void csboot_cmd_get_version(void);
static void csboot_cmd_write_memory(void);
static void csboot_cmd_read_memory(void);
static void csboot_cmd_erase_page(void);
static void csboot_cmd_erase_chip(void);
static void csboot_cmd_go(void);
static void csboot_cmd_write_protect(void);
static void csboot_cmd_write_unprotect(void);
static void csboot_cmd_readout_protect(void);
static void csboot_cmd_readout_unprotect(void);


void comm_timeout_check(void)
{
    uint32_t timeout = 0;
    Function_Pointer jump_to_address;
        
    timeout = systick_counter;
    while((systick_counter - timeout) < (CSBOOT_TIMEOUT_CHECK + 50)  && (comm_rx_index == 0))
    {
        /* Wait time until rx data received */
        if((systick_counter - timeout) >= CSBOOT_TIMEOUT_CHECK)
        {
            if(((*(__IO uint32_t*)FLASH_APP_ADDR)&0x2FFE0000)==0x20000000)
            {
                csboot_handle.csboot_comm_deinit();
                jump_to_address = (Function_Pointer)(*(__IO uint32_t *)(FLASH_APP_ADDR + 4U));
                __set_MSP(*(__IO uint32_t*) FLASH_APP_ADDR);
                jump_to_address();
            }
        }
    }
}

/**@brief       Executes the command received.
 * 
 * @param[in]   packet_status: Specifies the status of received packet.
 * 
 * @return      None.
 */
void csboot_command_proess(uint8_t packet_status)
{
    comm_tx_buffer[0] = packet_status;
    comm_tx_buffer[1] = COMM_FRAME_HEAD;
    comm_tx_buffer[2] = 0x01;
    comm_tx_buffer[3] = CMD_EXECUTE_SUCCESS;
    
    if(packet_status == COMM_PACKET_ACK)
    {
        switch(comm_rx_buffer[3])
        {
            case CSBOOT_COMMAND_GET_ID:
                csboot_cmd_get_id();
            break;
            
            case CSBOOT_COMMAND_GET_VERSION:
                csboot_cmd_get_version();
            break;
            
            case CSBOOT_COMMAND_WRITE_MEMORY:
                csboot_cmd_write_memory();
            break;
            
            case CSBOOT_COMMAND_READ_MEMORY:
                csboot_cmd_read_memory();
            break;
            
            case CSBOOT_COMMAND_ERASE_PAGE:
                csboot_cmd_erase_page();
            break;   

            case CSBOOT_COMMAND_ERASE_CHIP:
                csboot_cmd_erase_chip();
            break;
            
            case CSBOOT_COMMAND_GO:
                csboot_cmd_go();
            break;
            
            case CSBOOT_COMMAND_WRITE_PROTECT:
                csboot_cmd_write_protect();
            break;
            
            case CSBOOT_COMMAND_WRITE_UNPROTECT:
                csboot_cmd_write_unprotect();
            break;
            
            case CSBOOT_COMMAND_READOUT_PROTECT:
                csboot_cmd_readout_protect();
            break;
            
            case CSBOOT_COMMAND_READOUT_UNPROTECT:
                csboot_cmd_readout_unprotect();
            break;
            
            default:
                comm_tx_buffer[3] = CMD_EXECUTE_NOTSUPPORT;
            break;
        }
                
    }
    else
    {
        comm_tx_buffer[3] = CMD_EXECUTE_FAIL;
    }
    return;
}


static void csboot_cmd_get_id(void)
{
    comm_tx_buffer[2] = 0x05;
    *(uint32_t *)&comm_tx_buffer[4] = *(const uint32_t *)chip_id;
    return;
}


static void csboot_cmd_get_version(void)
{
    comm_tx_buffer[2] = 0x05;
    *(uint32_t *)&comm_tx_buffer[4] = *(const uint32_t *)boot_version;
    return;
}


static void csboot_cmd_write_memory(void)
{
    memory_addr = *(uint32_t *)&comm_rx_buffer[4];
    memory_len = (comm_rx_buffer[2] - 5) >> 1;

    if((memory_addr >= RAM_ADDR_START) && (memory_addr <= RAM_ADDR_END))
    {
        cmd_execute_status = ram_write(memory_addr, memory_len, (uint16_t *)&comm_rx_buffer[8]);
    }
    else if((memory_addr & OB_ADDR_MASK) == 0x1FFFF800)
    {
        cmd_execute_status = ob_write(memory_addr, memory_len, (uint16_t *)&comm_rx_buffer[8]);
        if(cmd_execute_status == CMD_EXECUTE_SUCCESS)
        {
            chip_reset_flag = 0xFF;
        }
    }
    else if((memory_addr & FLASH_ADDR_MASK) == 0x08000000)
    {
        cmd_execute_status = flash_write(memory_addr, memory_len, (uint16_t *)&comm_rx_buffer[8]);
    }
    else
    {
        cmd_execute_status = CMD_EXECUTE_PARAMETER_ERROR;
    }

    comm_tx_buffer[3] = cmd_execute_status;
    return;
}

static void csboot_cmd_read_memory(void)
{
    comm_tx_buffer[2] = comm_rx_buffer[8] + 1;
    
    memory_addr = *(uint32_t *)&comm_rx_buffer[4];
    memory_len = comm_rx_buffer[8];
    
    if((memory_addr >= RAM_ADDR_START) && (memory_addr <= RAM_ADDR_END))
    {
        cmd_execute_status = ram_read(memory_addr, memory_len, (uint8_t *)&comm_tx_buffer[4]);
    }
    else if((memory_addr & OB_ADDR_MASK) == 0x1FFFF800)
    {
        cmd_execute_status = ob_read(memory_addr, memory_len, (uint8_t *)&comm_tx_buffer[4]);
    }
    else if((memory_addr & FLASH_ADDR_MASK) == 0x08000000)
    {
        cmd_execute_status = flash_read(memory_addr, memory_len, (uint8_t *)&comm_tx_buffer[4]);
    }
    else
    {
        cmd_execute_status = CMD_EXECUTE_PARAMETER_ERROR;
    }
    
    if(cmd_execute_status != CMD_EXECUTE_SUCCESS)
    {
        comm_tx_buffer[2] = 0x01;
    }
    comm_tx_buffer[3] = cmd_execute_status;
    
    return;
}

static void csboot_cmd_erase_page(void)
{
    memory_addr = *(uint32_t *)&comm_rx_buffer[4];
    
    if((memory_addr & FLASH_ADDR_MASK) == 0x08000000)
    {
        cmd_execute_status = flash_page_erase(memory_addr);
    }
    else
    {
        cmd_execute_status = CMD_EXECUTE_PARAMETER_ERROR;
    }
    
    comm_tx_buffer[3] = cmd_execute_status;
    
    return;
}

static void csboot_cmd_erase_chip(void)
{
    if(flash_chip_erase() != MEM_ERROR_NONE)
    {
        comm_tx_buffer[3] = CMD_EXECUTE_FAIL;
    }
    return;
}

static void csboot_cmd_go(void)
{
    Function_Pointer jump_to_address;
    
    memory_addr = *(uint32_t *)&comm_rx_buffer[4];

    if(((memory_addr >= RAM_ADDR_START) && (memory_addr <= RAM_ADDR_END)) || ((memory_addr & FLASH_ADDR_MASK) == 0x08000000))
    {
        csboot_handle.csboot_comm_deinit();
        jump_to_address = (Function_Pointer)(*(__IO uint32_t *)(memory_addr + 4U));
        __set_MSP(*(__IO uint32_t*) memory_addr);
        jump_to_address();
    }
    else
    {
        comm_tx_buffer[3] = CMD_EXECUTE_PARAMETER_ERROR;
    }
    return;
}

static void csboot_cmd_write_protect(void)
{
    if(ob_write(OB_WRP_ADDR_START, 4, (uint16_t *)&comm_rx_buffer[4]) != MEM_ERROR_NONE)
    {
        comm_tx_buffer[3] = CMD_EXECUTE_FAIL;
    }
    else
    {
        chip_reset_flag = 0xFF;
    }
    return;
}

static void csboot_cmd_write_unprotect(void)
{
    ob_data[0] = 0x00FF;
    ob_data[1] = 0x00FF;
    ob_data[2] = 0x00FF;
    ob_data[3] = 0x00FF;
    if(ob_write(OB_WRP_ADDR_START, 4, (uint16_t *)ob_data) != MEM_ERROR_NONE)
    {
        comm_tx_buffer[3] = CMD_EXECUTE_FAIL;
    }
    else
    {
        chip_reset_flag = 0xFF;
    }
    return;
}

static void csboot_cmd_readout_protect(void)
{
    ob_data[0] = 0xFF00;
    if(ob_write(OB_RDP_ADDR_START, 1, (uint16_t *)ob_data) != MEM_ERROR_NONE)
    {
        comm_tx_buffer[3] = CMD_EXECUTE_FAIL;
    }
    else
    {
        chip_reset_flag = 0xFF;
    }
    return;
}

static void csboot_cmd_readout_unprotect(void)
{
    ob_data[0] = 0x00A5;
    if(ob_write(OB_RDP_ADDR_START, 1, (uint16_t *)ob_data) != MEM_ERROR_NONE)
    {
        comm_tx_buffer[3] = CMD_EXECUTE_FAIL;
    }
    else
    {
        chip_reset_flag = 0xFF;
    }
    return;
}

