/***************************************************************************//**
 * @file        cs32f10x_can.h
 * @version     V2.0.1
 * @author      Software Development
 * @brief       This file contains the definition of CAN function parameters and function interface declaration.
 *              
 * @copyright   Copyright (C) Software Development. All rights reserved.
****************************************************************************/

#ifndef __CS32F10X_CAN_H__
#define __CS32F10X_CAN_H__

#ifdef __cplusplus
extern "C" {
#endif

/*****************************************************************************
 * @includes 
*****************************************************************************/

#include "cs32f10x.h"

/** @addtogroup CAN
  Configuration of the M3 Processor and Core Peripherals:
###Features: 
    - interrupt numbers
    - registers and bit fields
    - peripheral base address
    - peripheral ID
    - Peripheral definitions
  @{
*/

/** 
  * @brief  CAN basic configuration structure definition
  */
typedef struct
{
    uint32_t can_mode;          /*!< CAN work mode,it can be one of the following values:@ref CAN_work_mode */

    uint32_t baudrate_pdiv;     /*!< CAN baud rate prescaler division */

    uint32_t time_change;       /*!< Specifies the maximum number of time unit the CAN module
                                     could be changed lengther or shorter. it can be one of the following values:
                                     @ref CAN_time_change */

    uint32_t time_seg1;         /*!< Time segment 1. it can be one of the following values:@ref CAN_time_segment_1 */

    uint32_t time_seg2;         /*!< Time segment 2. it can be one of the following values:@ref CAN_time_segment_2 */

    enable_state_t time_trigger; /*!< Time trigger communication mode.This parameter can be ENABLE or DISABLE */

    enable_state_t auto_busoff;  /*!< Automatic bus-off management. This parameter can be ENABLE or DISABLE */

    enable_state_t auto_active;   /*!< Automatic activation. This parameter can be ENABLE or DISABLE */

    enable_state_t no_retransmit; /*!< No automotic retransmission. This parameter can be ENABLE or DISABLE */

    enable_state_t rx_overwrite;  /*!< Receive FIFO over-write mode.This parameter can be ENABLE or DISABLE */

    enable_state_t tx_priority;   /*!< Transmit FIFO priority. This parameter can be ENABLE or DISABLE */
    
}can_config_t;

/** 
  * @brief  CAN filter structure definition
  */
typedef struct
{
    uint32_t filter_identifier;     /*!< Specifies the filter identification value.
                                    This parameter can be a value between 0x00000000 to 0xFFFFFFFF*/

    uint32_t filter_mask;           /*!< Specifies the filter mask value.
                                    This parameter can be a value between 0x00000000 to 0xFFFFFFFF*/

    uint8_t filter_fifo_connect;   /*!< FIFO0 or FIFO1 connect to the filter.
                                    it can be one of the following values:@ref CAN_filter_fifo */

    uint8_t filter_group;           /*!< Specifies the filter group to be used.  it can be between 0 and 13. */

    uint8_t filter_mode;            /*!< Specifies the filter mode.
                                    it can be one of the following values:@ref CAN_filter_mode */

    uint8_t filter_width;           /*!< Specifies the filter width.
                                    it can be one of the following values:@ref CAN_filter_width */

    enable_state_t filter_active;   /*!< Active filter. This parameter can be ENABLE or DISABLE */
                                    
}can_filter_config_t;

/** 
  * @brief  CAN tx message mailbox definition
  */
typedef struct
{
    uint32_t std_id;                /*!< The standard identifier.
                                    This parameter can be a value between 0 to 0x7FF (11 bits) */

    uint32_t ext_id;                /*!< The extended identifier.
                                    This parameter can be a value between 0 to 0x1FFFFFFF (29 bits) */

    uint8_t id_select;              /*!< Select either the standard identifier or extended identifier.
                                    It can be one of the following values:@ref CAN_id */

    uint8_t remote_txreq;           /*!< Select frame type to be transmitted.
                                    It can be one of the following values:@ref CAN_remote_txreq */

    uint8_t data_length;            /*!< The length of data.
                                    This parameter can be a value between 0 to 8 */

    uint8_t data[8];                /*!< mailbox transmit data */
                                    
}can_tx_msg_t;


/** 
  * @brief  CAN rx message mailbox definition
  */
typedef struct
{
    uint32_t std_id;                /*!< The standard identifier.
                                    This parameter can be a value between 0 to 0x7FF (11 bits) */

    uint32_t ext_id;                /*!< The extended identifier.
                                    This parameter can be a value between 0 to 0x1FFFFFFF (29 bits) */

    uint8_t id_select;              /*!< Select either the standard identifier or extended identifier.
                                    It can be one of the following values:@ref CAN_id */

    uint8_t remote_txreq;           /*!< Select frame type to be transmitted.
                                    It can be one of the following values:@ref CAN_remote_txreq */

    uint8_t data_length;            /*!< The length of data.
                                    This parameter can be a value between 0 to 8 */

    uint8_t data[8];                /*!< Mailbox transmit data */

    uint8_t match_Info;             /*!< Information choose control.
                                    This parameter can be a value between 0 to 0xFF */
    
}can_rx_msg_t;

/*****************************************************************************
 * @definitions 
*****************************************************************************/

/*!< Check CAN list */
#define CAN_LIST_CHECK(CANx)        ((CANx) == CAN1)

/** @defgroup CAN_work_mode
  * @{
  */

#define CAN_MODE_NORMAL             ((uint32_t)0x00000000)  /*!< normal mode */
#define CAN_MODE_LOOPBACK           ((uint32_t)0x40000000)  /*!< loopback mode */
#define CAN_MODE_SILENT             ((uint32_t)0x80000000)  /*!< silent mode */
#define CAN_MODE_LOOPBACK_SILENT    ((uint32_t)0xC0000000)  /*!< loopback and silent mode */

/*!< Check CAN work mode */
#define CAN_MODE_CHECK(mode)        (((mode) == CAN_MODE_NORMAL) || \
                                     ((mode) == CAN_MODE_LOOPBACK) || \
                                     ((mode) == CAN_MODE_SILENT) || \
                                     ((mode) == CAN_MODE_LOOPBACK_SILENT))
/**
  * @}
  */

/*!< Check CAN prescaler value */
#define CAN_PDIV_CHECK(pdiv)        (((pdiv) >= 1) && ((pdiv) <= 1024))

/** @defgroup CAN_time_change
  * @{
  */

#define CAN_TIME_CHANGE_UNIT_1      ((uint32_t)0x00000000)  /*!< 1 time unit */
#define CAN_TIME_CHANGE_UNIT_2      ((uint32_t)0x01000000)  /*!< 2 time unit */
#define CAN_TIME_CHANGE_UNIT_3      ((uint32_t)0x02000000)  /*!< 3 time unit */
#define CAN_TIME_CHANGE_UNIT_4      ((uint32_t)0x03000000)  /*!< 4 time unit */

#define CAN_TIME_CHANGE_UNIT_CHECK(value)   (((value) == CAN_TIME_CHANGE_UNIT_1) || \
                                              ((value) == CAN_TIME_CHANGE_UNIT_2) || \
                                              ((value) == CAN_TIME_CHANGE_UNIT_3) || \
                                              ((value) == CAN_TIME_CHANGE_UNIT_4))
/**
  * @}
  */

/** @defgroup CAN_time_segment_1
  * @{
  */
#define CAN_TS1_UNIT_1              ((uint32_t)0x00000000)
#define CAN_TS1_UNIT_2              ((uint32_t)0x00010000)
#define CAN_TS1_UNIT_3              ((uint32_t)0x00020000)
#define CAN_TS1_UNIT_4              ((uint32_t)0x00030000)
#define CAN_TS1_UNIT_5              ((uint32_t)0x00040000)
#define CAN_TS1_UNIT_6              ((uint32_t)0x00050000)
#define CAN_TS1_UNIT_7              ((uint32_t)0x00060000)
#define CAN_TS1_UNIT_8              ((uint32_t)0x00070000)
#define CAN_TS1_UNIT_9              ((uint32_t)0x00080000)
#define CAN_TS1_UNIT_10             ((uint32_t)0x00090000)
#define CAN_TS1_UNIT_11             ((uint32_t)0x000A0000)
#define CAN_TS1_UNIT_12             ((uint32_t)0x000B0000)
#define CAN_TS1_UNIT_13             ((uint32_t)0x000C0000)
#define CAN_TS1_UNIT_14             ((uint32_t)0x000D0000)
#define CAN_TS1_UNIT_15             ((uint32_t)0x000E0000)
#define CAN_TS1_UNIT_16             ((uint32_t)0x000F0000)


/*!< Check CAN time segment1 value */
#define CAN_TS1_UNIT_CHECK(value)   (((value) >> 16) <= 0xF)

/**
  * @}
  */

/** @defgroup CAN_time_segment_2
  * @{
  */
#define CAN_TS2_UNIT_1              ((uint32_t)0x00000000)
#define CAN_TS2_UNIT_2              ((uint32_t)0x00100000)
#define CAN_TS2_UNIT_3              ((uint32_t)0x00200000)
#define CAN_TS2_UNIT_4              ((uint32_t)0x00300000)
#define CAN_TS2_UNIT_5              ((uint32_t)0x00400000)
#define CAN_TS2_UNIT_6              ((uint32_t)0x00500000)
#define CAN_TS2_UNIT_7              ((uint32_t)0x00600000)
#define CAN_TS2_UNIT_8              ((uint32_t)0x00700000)

/*!< Check CAN time segment2 value */
#define CAN_TS2_UNIT_CHECK(value)   (((value) >> 20) <= 0x7)


/**
  * @}
  */

/** @defgroup CAN_filter_fifo
  * @{
  */

#define CAN_FILTER_CONNECT_FIFO0        ((uint8_t)0x00)
#define CAN_FILTER_CONNECT_FIFO1        ((uint8_t)0x01)

/*!< Check CAN filter connect to the FIFOx */
#define CAN_FILTER_CONNECT_FIFO_CHECK(value)    (((value) == CAN_FILTER_CONNECT_FIFO0) || \
                                                 ((value) == CAN_FILTER_CONNECT_FIFO1))
/**
  * @}
  */


/** @defgroup CAN_filter_mode
  * @{
  */

#define CAN_FILTER_MODE_IDENTIFY_MASK       ((uint8_t)0x00)     /*!< identifies mask mode */
#define CAN_FILTER_MODE_IDENTIFY_LIST       ((uint8_t)0x01)     /*!< identifies list mode */

/*!< Check CAN filter mode */
#define CAN_FILTER_MODE_CHECK(mode)         (((mode) == CAN_FILTER_MODE_IDENTIFY_MASK) || \
                                             ((mode) == CAN_FILTER_MODE_IDENTIFY_LIST))

/**
  * @}
  */
    
/** @defgroup CAN_filter_width
  * @{
  */

#define CAN_FILTER_WIDTH_16BIT              ((uint8_t)0x00)     /*!< 2* (16 bits) filter */
#define CAN_FILTER_WIDTH_32BIT              ((uint8_t)0x01)     /*!< 1* (32 bits) filter */

/*!< Check CAN filter width */
#define CAN_FILTER_WIDTH_CHECK(width)       (((width) == CAN_FILTER_WIDTH_16BIT) || \
                                             ((width) == CAN_FILTER_WIDTH_32BIT))

/**
  * @}
  */

/*!< Check CAN filter group */
#define CAN_FILTER_GROUP_CHECK(group)       ((group) <= 13)


/** @defgroup CAN_id
  * @{
  */

#define CAN_ID_STANDARD                     ((uint32_t)0x00000000)
#define CAN_ID_EXTEND                       ((uint32_t)0x00000004)

/*!< Check CAN identifier */
#define CAN_ID_CHECK(id)                    (((id) == CAN_ID_STANDARD) || ((id) == CAN_ID_EXTEND))
/**
  * @}
  */

/** @defgroup CAN_remote_txreq
  * @{
  */

#define CAN_RMOTE_TX_REQ_DATA               ((uint32_t)0x00000000)
#define CAN_RMOTE_TX_REQ_REMOTE             ((uint32_t)0x00000002)

/*!< Check CAN identifier */
#define CAN_RMOTE_TX_REQ_CHECK(type)    (((type) == CAN_RMOTE_TX_REQ_DATA) || ((type) == CAN_RMOTE_TX_REQ_REMOTE))
/**
  * @}
  */

/*!< Check CAN message data length */
#define CAN_MSG_DATA_LENGTH_CHECK(len)      ((len) <= (uint8_t)0x08)

/** @defgroup CAN_tx_mailbox_number
  * @{
  */

#define CAN_TX_MAILBOX_NUM0                     ((uint8_t)0x00)
#define CAN_TX_MAILBOX_NUM1                     ((uint8_t)0x01)
#define CAN_TX_MAILBOX_NUM2                     ((uint8_t)0x02)

/*!< Check CAN TX MAILBOX number */
#define CAN_TX_MAILBOX_NUMBER_CHECK(num)        (((num) == CAN_TX_MAILBOX_NUM0) || \
                                                ((num) == CAN_TX_MAILBOX_NUM1) || \
                                                ((num) == CAN_TX_MAILBOX_NUM2))

/**
  * @}
  */


/** @defgroup CAN_rx_fifo
  * @{
  */

#define CAN_RECEIVE_FIFO_0                       ((uint8_t)0x00)
#define CAN_RECEIVE_FIFO_1                       ((uint8_t)0x01)

/*!< Check CAN RX FIFO number */
#define CAN_RECEIVE_FIFO_CHECK(num)              (((num) == CAN_RECEIVE_FIFO_0) || \
                                                 ((num) == CAN_RECEIVE_FIFO_1))

/**
  * @}
  */


/** @defgroup CAN_operation_mode
  * @{
  */

#define CAN_OP_MODE_SLEEP                          ((uint8_t)0x00)
#define CAN_OP_MODE_INITIAL                        ((uint8_t)0x01)
#define CAN_OP_MODE_NORMAL                         ((uint8_t)0x02)

/*!< Check CAN operation mode */
#define CAN_OP_MODE_CHECK(mode)                     (((mode) == CAN_OP_MODE_SLEEP) || \
                                                    ((mode) == CAN_OP_MODE_INITIAL) || \
                                                    ((mode) == CAN_OP_MODE_NORMAL))

/**
  * @}
  */

/** @defgroup CAN_flag_definition
  * note:
  * [bit 27]    : TXSTS flag
  * [bit 26]    : RXFF1 flag
  * [bit 25]    : RXFF0 flag
  * [bit 24]    : MSTS flag
  * [bit 23:20] : ERRSTS flag
  * [bit 29:28] : 01 -- used for can_flag_status_get()
  *               11 -- used for can_flag_status_get() and can_flag_clear()
  * @{
  */

/*!< CAN flags in TXSTS register */
#define CAN_FLAG_IN_TXSTS_REG                       ((uint32_t)0x08000000)

#define CAN_FLAG_MRQCP0             ((uint32_t)0x38000001) /*!< Mailbox0 Request completed flag */
#define CAN_FLAG_MRQCP1             ((uint32_t)0x38000100) /*!< Mailbox1 Request completed flag */
#define CAN_FLAG_MRQCP2             ((uint32_t)0x38010000) /*!< Mailbox2 Request completed flag */

/* CAN flags in RXFF1 register */
#define CAN_FLAG_IN_RXFF1_REG                       ((uint32_t)0x04000000)

#define CAN_FLAG_FIFOMP1            ((uint32_t)0x14000003) /*!< FIFO 1 message pengding */
#define CAN_FLAG_FIFOFULL1          ((uint32_t)0x34000008) /*!< FIFO 1 is full */
#define CAN_FLAG_FIFOOVR1           ((uint32_t)0x34000010) /*!< FIFO 1 overrun */


/*!< CAN flags in RXFF0 register */
#define CAN_FLAG_IN_RXFF0_REG                       ((uint32_t)0x02000000)

#define CAN_FLAG_FIFOMP0            ((uint32_t)0x12000003) /*!< FIFO 0 message pengding */
#define CAN_FLAG_FIFOFULL0          ((uint32_t)0x32000008) /*!< FIFO 0 is full */
#define CAN_FLAG_FIFOOVR0           ((uint32_t)0x32000010) /*!< FIFO 0 overrun */

/*!< CAN flags in MSTS register */
#define CAN_FLAG_IN_MSTS_REG                        ((uint32_t)0x01000000)

#define CAN_FLAG_SLEEPF             ((uint32_t)0x31000012) /*!< Sleep flag  */
#define CAN_FLAG_WKUI               ((uint32_t)0x31000008) /*!< Wakeup flag  */

/*!< CAN flags in ERRSTS register */
#define CAN_FLAG_IN_ERRSTS_REG                      ((uint32_t)0x00F00000)

#define CAN_FLAG_ERRWF              ((uint32_t)0x10F00001) /*!< Error Warning Flag */
#define CAN_FLAG_ERRPF              ((uint32_t)0x10F00002) /*!< Error passive flag */
#define CAN_FLAG_BOFF               ((uint32_t)0x10F00004) /*!< Bus-off flag */
#define CAN_FLAG_LEC                ((uint32_t)0x30F00070) /*!< Last error code Flag */

/*!< Check CAN flag get */
#define CAN_GET_FLAG_CHECK(flag)    (((flag) == CAN_FLAG_LEC)  || ((flag) == CAN_FLAG_BOFF)   || \
                                   ((flag) == CAN_FLAG_ERRPF)  || ((flag) == CAN_FLAG_ERRWF)   || \
                                   ((flag) == CAN_FLAG_WKUI)  || ((flag) == CAN_FLAG_SLEEPF)  || \
                                   ((flag) == CAN_FLAG_FIFOOVR0)  || ((flag) == CAN_FLAG_FIFOFULL0)  || \
                                   ((flag) == CAN_FLAG_FIFOMP0) || ((flag) == CAN_FLAG_FIFOOVR1)   || \
                                   ((flag) == CAN_FLAG_FIFOFULL1) || ((flag) == CAN_FLAG_FIFOMP1) || \
                                   ((flag) == CAN_FLAG_MRQCP2)|| ((flag) == CAN_FLAG_MRQCP1) || \
                                   ((flag) == CAN_FLAG_MRQCP0 ))

#define CAN_CLEAR_FLAG_CHECK(flag)  (((flag) == CAN_FLAG_LEC)  || ((flag) == CAN_FLAG_WKUI)   || \
                                   ((flag) == CAN_FLAG_SLEEPF)  || ((flag) == CAN_FLAG_FIFOOVR0)   || \
                                   ((flag) == CAN_FLAG_FIFOFULL0)  || ((flag) == CAN_FLAG_FIFOOVR1)  || \
                                   ((flag) == CAN_FLAG_FIFOFULL1)  || ((flag) == CAN_FLAG_MRQCP2)  || \
                                   ((flag) == CAN_FLAG_MRQCP1) || ((flag) == CAN_FLAG_MRQCP0))

/**
  * @}
  */



/**
  * @brief CAN error codes definition
  */
#define CAN_ERRCODE_NOERR                       ((uint32_t)0x00000000)
#define CAN_ERRCODE_BIT_FILLING                 ((uint32_t)0x00000010)
#define CAN_ERRCODE_FORMAT                      ((uint32_t)0x00000020)
#define CAN_ERRCODE_ACK                         ((uint32_t)0x00000030)
#define CAN_ERRCODE_BIT_RECESSIVE               ((uint32_t)0x00000040)
#define CAN_ERRCODE_BIT_DOMINANT                ((uint32_t)0x00000050)
#define CAN_ERRCODE_CRC                         ((uint32_t)0x00000060)
#define CAN_ERRCODE_SOFTWARE_SET                ((uint32_t)0x00000070)

/**
  * @brief CAN interrupt definition
  */
#define CAN_INTR_TX_EMPTY               ((uint32_t)0x00000001) /*!< Transmit empty interrupt enable of mailbox */
#define CAN_INTR_FF0_MSG_PENDING        ((uint32_t)0x00000002) /*!< FIFO0 message pending interrupt enable */
#define CAN_INTR_FF0_FULL               ((uint32_t)0x00000004) /*!< FIFO0 full interrupt enable */
#define CAN_INTR_FF0_OVRRUN             ((uint32_t)0x00000008) /*!< FIFO0 overrun interrupt enable */
#define CAN_INTR_FF1_MSG_PENDING        ((uint32_t)0x00000010) /*!< FIFO1 message pending interrupt enable */
#define CAN_INTR_FF1_FULL               ((uint32_t)0x00000020) /*!< FIFO1 full interrupt enable */
#define CAN_INTR_FF1_OVERRUN            ((uint32_t)0x00000040) /*!< FIFO1 overrun interrupt enable */
#define CAN_INTR_ERR_WARNING            ((uint32_t)0x00000100) /*!< Error warning interrupt enable */
#define CAN_INTR_ERR_PASSIVE            ((uint32_t)0x00000200) /*!< Error passive interrupt enable */
#define CAN_INTR_BUS_OFF                ((uint32_t)0x00000400) /*!< Bus-off interrupt enable */
#define CAN_INTR_LAST_ERR_NUMBER        ((uint32_t)0x00000800) /*!< Last error number interrupt enable */
#define CAN_INTR_ERROR                  ((uint32_t)0x00008000) /*!< Error interrupt enable */
#define CAN_INTR_WAKEUP                 ((uint32_t)0x00010000) /*!< Wakeup interrupt enable */
#define CAN_INTR_SLEEP                  ((uint32_t)0x00020000) /*!< Sleep interrupt enable */

/*!< Check CAN interrupt flag */
#define CAN_INTR_FLAG_CHECK(int)        (((int) == CAN_INTR_TX_EMPTY)  || ((int) == CAN_INTR_FF0_MSG_PENDING)   || \
                                        ((int) == CAN_INTR_FF0_FULL)  || ((int) == CAN_INTR_FF0_OVRRUN)   || \
                                        ((int) == CAN_INTR_FF1_MSG_PENDING)  || ((int) == CAN_INTR_FF1_FULL)  || \
                                        ((int) == CAN_INTR_FF1_OVERRUN)  || ((int) == CAN_INTR_ERR_WARNING)  || \
                                        ((int) == CAN_INTR_ERR_PASSIVE) || ((int) == CAN_INTR_BUS_OFF)   || \
                                        ((int) == CAN_INTR_LAST_ERR_NUMBER) || ((int) == CAN_INTR_ERROR) || \
                                        ((int) == CAN_INTR_WAKEUP)|| ((int) == CAN_INTR_SLEEP))


#define CAN_INTR_CLEAR_CHECK(int)       (((int) == CAN_INTR_TX_EMPTY)  || ((int) == CAN_INTR_SLEEP)   || \
                                        ((int) == CAN_INTR_FF0_FULL)  || ((int) == CAN_INTR_FF0_OVRRUN)   || \
                                        ((int) == CAN_INTR_WAKEUP)  || ((int) == CAN_INTR_FF1_FULL)  || \
                                        ((int) == CAN_INTR_FF1_OVERRUN)  || ((int) == CAN_INTR_ERR_WARNING)  || \
                                        ((int) == CAN_INTR_ERR_PASSIVE) || ((int) == CAN_INTR_BUS_OFF)   || \
                                        ((int) == CAN_INTR_LAST_ERR_NUMBER) || ((int) == CAN_INTR_ERROR))

/**
  * @brief CAN function register definition
  */
#define CAN_ENABLE_REG_DEBUG_WAKE               MCTR
#define CAN_ENABLE_REG_SOFTWARE_RESET           MCTR
#define CAN_ENABLE_REG_AUTO_BUSOFF              MCTR
#define CAN_ENABLE_REG_AUTO_WAKEUP              MCTR
#define CAN_ENABLE_REG_NO_RETRANSMIT            MCTR
#define CAN_ENABLE_REG_RX_OVRWRITE              MCTR
#define CAN_ENABLE_REG_TX_PRIORITY              MCTR

/**
  * @brief CAN function bit definition
  */
#define CAN_ENABLE_BIT_SOFTWARE_RESET           CAN_MCTR_SOFTRST
#define CAN_ENABLE_BIT_DEBUG_WAKE               ((uint32_t)0x00010000)
#define CAN_ENABLE_BIT_AUTO_BUSOFF              CAN_MCTR_AUTOFFM
#define CAN_ENABLE_BIT_AUTO_WAKEUP              CAN_MCTR_AUTOATV
#define CAN_ENABLE_BIT_NO_RETRANSMIT            CAN_MCTR_NAUTORT
#define CAN_ENABLE_BIT_RX_OVRWRITE              CAN_MCTR_RFOWRM
#define CAN_ENABLE_BIT_TX_PRIORITY              CAN_MCTR_TXFP




/*****************************************************************************
 * @functions
*****************************************************************************/


/**@brief       Reset CAN1 module registers to default value.
 *
 * @param[in]   None.
 *
 * @return      None.
 */
#define __CAN_DEF_INIT()                do{ \
                                                __RCU_APB1_PERI_RST_ENABLE(RCU_RST_CAN1); \
                                                __RCU_APB1_PERI_RST_DISABLE(RCU_RST_CAN1); \
                                          }while(0U)

/**@brief       Enables the specified CAN function.
 *
 * @param[in]   CANx:Pointer to a specified CAN peripheral. It only can be CAN1.
 *
 * @param[in]   FUNC: Specifies the CAN function, it can be one of the following values:
 *              SOFTWARE_RESET (software master reset)
 *              DEBUG_WAKE (debug wakeup)
 *              AUTO_BUSOFF (auto buss-off management)
 *              AUTO_WAKEUP (automatic wakeup mode)
 *              NO_RETRANSMIT (no automatic retransmission)
 *              RX_OVRWRITE (receive FIFO over-write mode)
 *              TX_PRIORITY (Transmit FIFO priority)
 *
 * @return      None.
 */
#define __CAN_FUNC_ENABLE(CANx,FUNC)   (CANx->CAN_ENABLE_REG_##FUNC |= CAN_ENABLE_BIT_##FUNC)

/**@brief       Disable the specified CAN function.
 *
 * @param[in]   CANx:Pointer to a specified CAN peripheral. It only can be CAN1.
 *
 * @param[in]   FUNC: Specifies the CAN function, it can be one of the following values:
 *              SOFTWARE_RESET (software master reset)
 *              DEBUG_WAKE (debug wakeup)
 *              AUTO_BUSOFF (auto buss-off management)
 *              AUTO_WAKEUP (automatic wakeup mode)
 *              NO_RETRANSMIT (no automatic retransmission)
 *              RX_OVRWRITE (receive FIFO over-write mode)
 *              TX_PRIORITY (Transmit FIFO priority)
 *
 * @return      None.
 */
#define __CAN_FUNC_DISABLE(CANx,FUNC)   (CANx->CAN_ENABLE_REG_##FUNC &= ~(CAN_ENABLE_BIT_##FUNC))

/**@brief       Get CAN last error code.
 *
 * @param[in]   CANx:Pointer to a specified CAN peripheral. It only can be CAN1.
 *
 * @return      It can be one of  the following values:
 *              CAN_ERRCODE_NOERR
 *              CAN_ERRCODE_BIT_FILLING
 *              CAN_ERRCODE_FORMAT
 *              CAN_ERRCODE_ACK
 *              CAN_ERRCODE_BIT_RECESSIVE
 *              CAN_ERRCODE_BIT_DOMINANT
 *              CAN_ERRCODE_SOFTWARE_SET
 *              CAN_ERRCODE_CRC
 */
#define __CAN_LAST_ERRCODE_GET(CANx)    (CANx->ERRSTS & CAN_ERRSTS_LEC)

/**@brief       Get CAN receive error counter.
 *
 * @param[in]   CANx:Pointer to a specified CAN peripheral. It only can be CAN1.
 *
 * @return      The value of receive error counter.
 */
#define __CAN_RX_ERRCOUNTER_GET(CANx)     ((CANx->ERRSTS & CAN_ERRSTS_RFC) >> 24)

/**@brief       Get CAN transmit error counter.
 *
 * @param[in]   CANx:Pointer to a specified CAN peripheral. It only can be CAN1.
 *
 * @return      LSB of 9-bit transmit error counter.
 */
#define __CAN_LSB_TX_ERRCOUNTER_GET(CANx)   ((CANx->ERRSTS & CAN_ERRSTS_TEC) >> 16)

/**@brief       Enable the CAN interrupt.
 *
 * @param[in]   CANx:Pointer to a specified CAN peripheral. It only can be CAN1.
 *
 * @param[in]   INT:specifies the CAN interrupt sources to be enabled. It can be any combination of the following values:
 *              CAN_INTR_TX_EMPTY
 *              CAN_INTR_FF0_MSG_PENDING
 *              CAN_INTR_FF0_FULL
 *              CAN_INTR_FF0_OVRRUN
 *              CAN_INTR_FF1_MSG_PENDING
 *              CAN_INTR_FF1_FULL
 *              CAN_INTR_FF1_OVERRUN
 *              CAN_INTR_ERR_WARNING
 *              CAN_INTR_ERR_PASSIVE
 *              CAN_INTR_BUS_OFF
 *              CAN_INTR_LAST_ERR_NUMBER
 *              CAN_INTR_ERROR
 *              CAN_INTR_WAKEUP
 *              CAN_INTR_SLEEP
 *
 * @return      None.
 */
#define __CAN_INTR_ENABLE(CANx, INT)        (CANx->INTR |= (INT))

/**@brief       Disable the CAN interrupt.
 *
 * @param[in]   CANx:Pointer to a specified CAN peripheral. It only can be CAN1.
 *
 * @param[in]   INT:specifies the CAN interrupt sources to be enabled. It can be any combination of the following values:
 *              CAN_INTR_TX_EMPTY
 *              CAN_INTR_FF0_MSG_PENDING
 *              CAN_INTR_FF0_FULL
 *              CAN_INTR_FF0_OVRRUN
 *              CAN_INTR_FF1_MSG_PENDING
 *              CAN_INTR_FF1_FULL
 *              CAN_INTR_FF1_OVERRUN
 *              CAN_INTR_ERR_WARNING
 *              CAN_INTR_ERR_PASSIVE
 *              CAN_INTR_BUS_OFF
 *              CAN_INTR_LAST_ERR_NUMBER
 *              CAN_INTR_ERROR
 *              CAN_INTR_WAKEUP
 *              CAN_INTR_SLEEP
 *
 * @return      None.
 */
#define __CAN_INTR_DISABLE(CANx, INT)        (CANx->INTR &= ~(INT))


/**@brief       Initializes the specified CAN peripheral according to the specified parameters in the
 *              ptr_can_config.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It can be CAN1. 
 * 
 * @param[in]   ptr_can_config: Pointer to a can_config_t structure containing the configuration for the specified
 *              CAN peripheral. 
 * 
 * @return      Status of function called, it could be ERROR or SUCCESS.
 */
error_status_t can_init(can_reg_t *ptr_can, can_config_t *ptr_can_config);

/**@brief       Config each ptr_can_config member with its default value.
 *
 * @param[in]   ptr_can_config: Pointer to a can_config_t structure containing the configuration for the specified
 *              CAN peripheral. 
 * 
 * @return      None.
 */
void can_config_struct_init(can_config_t *ptr_can_config);

/**@brief       Set can work mode.Only initialization mode could be set
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It can be CAN1.
 * 
 * @param[in]   mode: can work mode.It can be one of the following values: @ref CAN_work_mode
 * 
 * @return      Status of function called, it could be ERROR or SUCCESS.
 */
error_status_t can_work_mode_config(can_reg_t *ptr_can, uint32_t mode);

/**@brief       Set filter configuration.
 * 
 * @param[in]   ptr_can_filter: Pointer to a specified CAN filter.
 *
 * @return      Status of function called, it could be ERROR or SUCCESS.
 */
void can_filter_init(can_filter_config_t *ptr_can_filter);

/**@brief       Enable or disable the CAN time trigger communication mode. If enabled,
 *              time stamp could be send:TIME[15:0] in the last two bytes of the 8-byte
 *              message, TIME[7:0] in the byte 6 and TIME[15:8] in the byte 7.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It can be CAN1.
 * 
 * @param[in]   status: ENABLE or DISABLE.
 * 
 * @return      None.
 */
void can_time_trigger_enable_ctrl(can_reg_t *ptr_can, enable_state_t status);

/**@brief       Initializes the specified CAN transmission configuration according to the specified parameters
 *              in the ptr_msg.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It can be CAN1. 
 * 
 * @param[in]   ptr_msg: Pointer to a can_tx_msg_t structure containing the configuration for the specified
 *              CAN message. 
 * 
 * @return      The number of mailbox or CAN_TX_MAILBOXFULL.
 */
uint8_t can_message_send(can_reg_t *ptr_can, can_tx_msg_t *ptr_msg);

/**@brief       CAN tx status check according to the mailbox number.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It only can be CAN1. 
 * 
 * @param[in]   tx_mailbox: specifies the mailbox
 * 
 * @return      The status of the mailbox send.
 */
uint8_t can_tx_status_get(can_reg_t *ptr_can, uint8_t tx_mailbox);

/**@brief       CAN tx stop according to the mailbox number.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It can be CAN1. 
 * 
 * @param[in]   tx_mailbox: Specifies the mailbox.
 * 
 * @return      None.
 */
void can_tx_abort_set(can_reg_t *ptr_can, uint8_t tx_mailbox);

/**@brief       CAN receive message according to the rx_fifo.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It only can be CAN1. 
 * 
 * @param[in]   rx_fifo: Specifies the receive fifo number.
 *
 * @param[out]  ptr_msg: Pointer to a can_rx_msg_t structure containing the configuration for the specified
 *              CAN receive message.
 * 
 * @return      None.
 */
void can_message_receive(can_reg_t *ptr_can, uint8_t rx_fifo, can_rx_msg_t *ptr_msg);

/**@brief       CAN FIFO release.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It only can be CAN1. 
 * 
 * @param[in]   rx_fifo: Specifies the receive fifo number.
 *
 * @return      None.
 */
void can_rx_fifo_release(can_reg_t *ptr_can, uint8_t rx_fifo);

/**@brief       Get CAN FIFO remained message.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It only can be CAN1. 
 * 
 * @param[in]   rx_fifo: Specifies the receive fifo number.
 *
 * @return      The number of FIFO remains.
 */
uint8_t can_fifo_msg_num_get(can_reg_t *ptr_can, uint8_t rx_fifo);

/**@brief       Set CAN sleep mode.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It only can be CAN1. 
 * 
 * @return      The status of sleep mode.
 */
uint8_t can_sleep_set(can_reg_t *ptr_can);

/**@brief       Set CAN wakeup mode.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It only can be CAN1. 
 * 
 * @return      The status of wakeup mode.
 */
uint8_t can_wakeup_set(can_reg_t *ptr_can);

/**@brief       Set CAN operation mode:sleep,initialization and normal.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It only can be CAN1. 
 * 
 * @param[in]   mode: CAN operation mode. It can be one of the following values: @ref CAN_operation_mode
 *
 * @return      The status of mode switch.
 */
uint8_t can_op_mode_switch_ctrl(can_reg_t *ptr_can, uint8_t mode);

/**@brief       Get CAN flag according to registers.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It only can be CAN1. 
 * 
 * @param[in]   flag: Specifies the flag.
 *              CAN_FLAG_MRQCP0 (Mailbox0 Request completed flag)
 *              CAN_FLAG_MRQCP1 (Mailbox1 Request completed flag)
 *              CAN_FLAG_MRQCP2 (Mailbox2 Request completed flag)
 *              CAN_FLAG_FIFOMP1 (FIFO 1 message pengding)
 *              CAN_FLAG_FIFOFULL1 (FIFO 1 is full)
 *              CAN_FLAG_FIFOOVR1 (FIFO 1 overrun)
 *              CAN_FLAG_FIFOMP0 (FIFO 0 message pengding)
 *              CAN_FLAG_FIFOFULL0 (FIFO 0 is full)
 *              CAN_FLAG_FIFOOVR0 (FIFO 0 overrun)
 *              CAN_FLAG_SLEEPF ( Sleep flag)
 *              CAN_FLAG_WKUI (Wakeup flag)
 *              CAN_FLAG_ERRWF (Error Warning Flag)
 *              CAN_FLAG_ERRPF (Error passive flag)
 *              CAN_FLAG_BOFF (Bus-off flag)
 *              CAN_FLAG_LEC (Last error code Flag)
 *
 * @return      The status of flag.(SET or REST)
 */
bit_status_t can_flag_status_get(can_reg_t *ptr_can, uint32_t flag);

/**@brief       Clear CAN flag.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It only can be CAN1. 
 * 
 * @param[in]   flag: Specifies the flag.
 *              CAN_FLAG_MRQCP0 (Mailbox0 Request completed flag)
 *              CAN_FLAG_MRQCP1 (Mailbox1 Request completed flag)
 *              CAN_FLAG_MRQCP2 (Mailbox2 Request completed flag)
 *              CAN_FLAG_FIFOFULL1 (FIFO 1 is full)
 *              CAN_FLAG_FIFOOVR1 (FIFO 1 overrun)
 *              CAN_FLAG_FIFOFULL0 (FIFO 0 is full)
 *              CAN_FLAG_FIFOOVR0 (FIFO 0 overrun)
 *              CAN_FLAG_SLEEPF ( Sleep flag)
 *              CAN_FLAG_WKUI (Wakeup flag)
 *              CAN_FLAG_LEC (Last error code Flag)
 *
 * @return      The status of flag.(SET or REST)
 */
void can_flag_clear(can_reg_t *ptr_can, uint32_t flag);

/**@brief       Clear CAN interrupt flag.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It only can be CAN1. 
 * 
 * @param[in]   int_flag: Specifies the flag.
 *              CAN_INTR_TX_EMPTY
 *              CAN_INTR_FF0_MSG_PENDING
 *              CAN_INTR_FF0_FULL
 *              CAN_INTR_FF0_OVRRUN
 *              CAN_INTR_FF1_MSG_PENDING
 *              CAN_INTR_FF1_FULL
 *              CAN_INTR_FF1_OVERRUN
 *              CAN_INTR_ERR_WARNING
 *              CAN_INTR_ERR_PASSIVE
 *              CAN_INTR_BUS_OFF
 *              CAN_INTR_LAST_ERR_NUMBER
 *              CAN_INTR_ERROR
 *              CAN_INTR_WAKEUP
 *              CAN_INTR_SLEEP
 *
 * @return      The status of flag.(SET or RESET)
 */
bit_status_t can_int_status_get(can_reg_t *ptr_can, uint32_t int_flag);

/**@brief       Clear CAN interrupt flag.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It only can be CAN1. 
 * 
 * @param[in]   int_flag: Specifies the flag.
 *              CAN_INTR_TX_EMPTY
 *              CAN_INTR_FF0_FULL
 *              CAN_INTR_FF0_OVRRUN
 *              CAN_INTR_FF1_FULL
 *              CAN_INTR_FF1_OVERRUN
 *              CAN_INTR_ERR_WARNING
 *              CAN_INTR_ERR_PASSIVE
 *              CAN_INTR_BUS_OFF
 *              CAN_INTR_LAST_ERR_NUMBER
 *              CAN_INTR_ERROR
 *              CAN_INTR_WAKEUP
 *              CAN_INTR_SLEEP
 *
 * @return      The status of flag.(SET or RESET)
 */
void can_int_flag_clear(can_reg_t *ptr_can, uint32_t clear_flag);

/*@}*/ /* end of group CAN */

#ifdef __cplusplus
}
#endif


#endif


/*** (C) COPYRIGHT 2021 Software Development. ***/





