/***************************************************************************//**
 * @file        cs32f10x_gpio.h
 * @version     V2.0.1
 * @author      Software Development
 * @brief       This file contains the definition of GPIO function parameters and function interface declaration.
 *              
 * @copyright   Copyright (C) Software Development. All rights reserved.
****************************************************************************/


#ifndef __CS32F10X_GPIO_H__
#define __CS32F10X_GPIO_H__

#ifdef __cplusplus
extern "C" {
#endif


/*****************************************************************************
 * @includes 
*****************************************************************************/

#include "cs32f10x.h"


/** @addtogroup GPIO
  Configuration of the M3 Processor and Core Peripherals:
###Features: 
    - interrupt numbers
    - registers and bit fields
    - peripheral base address
    - peripheral ID
    - Peripheral definitions
  @{
*/

/** 
  * @enum gpio_speed_t 
  * @brief GPIO Output Maximum frequency enumeration    
  */
typedef enum
{
    GPIO_SPEED_MEDIUM   = 0x01,     /*!< I/O output speed: Medium 10 MHz */
    GPIO_SPEED_LOW      = 0x02,     /*!< I/O output speed: Low 2 MHz */
    GPIO_SPEED_HIGH     = 0x03      /*!< I/O output speed: High 50 MHz */
}gpio_speed_t;


/*****************************************************************************
 * @definitions 
*****************************************************************************/

/*!< Check GPIO port */
#define GPIO_ALL_PORT_CHECK(port)           (((port) == GPIOA) || ((port) == GPIOB) || ((port) == GPIOC) || \
                                            ((port) == GPIOD) || ((port) == GPIOE))

/** @defgroup GPIO_pin
  * @{
  */

#define GPIO_PIN_0                          ((uint16_t)0x0001)      /*!< select Pin 0 */
#define GPIO_PIN_1                          ((uint16_t)0x0002)      /*!< select Pin 1 */
#define GPIO_PIN_2                          ((uint16_t)0x0004)      /*!< select Pin 2 */
#define GPIO_PIN_3                          ((uint16_t)0x0008)      /*!< select Pin 3 */
#define GPIO_PIN_4                          ((uint16_t)0x0010)      /*!< select Pin 4 */
#define GPIO_PIN_5                          ((uint16_t)0x0020)      /*!< select Pin 5 */
#define GPIO_PIN_6                          ((uint16_t)0x0040)      /*!< select Pin 6 */
#define GPIO_PIN_7                          ((uint16_t)0x0080)      /*!< select Pin 7 */
#define GPIO_PIN_8                          ((uint16_t)0x0100)      /*!< select Pin 8 */
#define GPIO_PIN_9                          ((uint16_t)0x0200)      /*!< select Pin 9 */
#define GPIO_PIN_10                         ((uint16_t)0x0400)      /*!< select Pin 10 */
#define GPIO_PIN_11                         ((uint16_t)0x0800)      /*!< select Pin 11 */
#define GPIO_PIN_12                         ((uint16_t)0x1000)      /*!< select Pin 12 */
#define GPIO_PIN_13                         ((uint16_t)0x2000)      /*!< select Pin 13 */
#define GPIO_PIN_14                         ((uint16_t)0x4000)      /*!< select Pin 14 */
#define GPIO_PIN_15                         ((uint16_t)0x8000)      /*!< select Pin 15 */
#define GPIO_PIN_ALL                        ((uint16_t)0xFFFF)      /*!< All pins selected */

/*!< Check GPIO pin */
#define GPIO_PIN_CHECK(pin)                 ((pin & 0xFFFF) != 0) 
#define GPIO_PIN_LIST_CHECK(pin)            (((pin) == GPIO_PIN_0)  || ((pin) == GPIO_PIN_1)  || ((pin) == GPIO_PIN_2) || \
                                            ((pin) == GPIO_PIN_3)  || ((pin) == GPIO_PIN_4)  || ((pin) == GPIO_PIN_5)  || \
                                            ((pin) == GPIO_PIN_6)  || ((pin) == GPIO_PIN_7)  || ((pin) == GPIO_PIN_8)  || \
                                            ((pin) == GPIO_PIN_9)  || ((pin) == GPIO_PIN_10) || ((pin) == GPIO_PIN_11) || \
                                            ((pin) == GPIO_PIN_12) || ((pin) == GPIO_PIN_13) || ((pin) == GPIO_PIN_14) || \
                                            ((pin) == GPIO_PIN_15))

/**
  * @}
  */

/** @defgroup GPIO_mode_selection
  *
  * bit1:0 - SPEED
  *
  * bit3:2 - 00:OUT_PP,    01:OUT_OD,   10:AFPP,   11:AFOD
  *          00:IN_ANALOG, 01:IN_FLOAT, 10:IN_PU or IN_PD
  *
  * bit4   - 0 :IN_UP, 1:IN_DOWN
  * bit6   - 1:OUT_MODE, 0:IN_MODE
  *
  * @{
  */

/*!< In mode definition */
#define GPIO_MODE_IN_ANALOG                 0x00                /*!< GPIO Analog In Mode */
#define GPIO_MODE_IN_FLOAT                  0x04                /*!< GPIO input floating */
#define GPIO_MODE_IN_PU                     0x08                /*!< GPIO input pull up */
#define GPIO_MODE_IN_PD                     0x18                /*!< GPIO input pull down */
/*!< Out mode definition */
#define GPIO_MODE_OUT_PP(SPEED)             (0x40 + SPEED)      /*!< GPIO output push-pull mode */
#define GPIO_MODE_OUT_OD(SPEED)             (0x44 + SPEED)      /*!< GPIO output open drain */
#define GPIO_MODE_OUT_AFPP(SPEED)           (0x48 + SPEED)      /*!< GPIO output AF push-pull mode */
#define GPIO_MODE_OUT_AFOD(SPEED)           (0x4C + SPEED)      /*!< GPIO output AF open drain */

/*!< Check GPIO mode */
#define GPIO_MODE_CHECK(mode)               (((mode) == GPIO_MODE_IN_ANALOG) || ((mode) == GPIO_MODE_IN_FLOAT) || \
                                            ((mode) == GPIO_MODE_IN_PU) || ((mode) == GPIO_MODE_IN_PD) || \
                                            (((mode) <= 0x4F) && ((mode) >= 0x40)))

/**
  * @}
  */

/** @defgroup GPIO_event_port
  * @{
  */

#define GPIO_EXTI_EVEVT_PORT_GPIOA               ((uint8_t)0x0)
#define GPIO_EXTI_EVEVT_PORT_GPIOB               ((uint8_t)0x1)
#define GPIO_EXTI_EVEVT_PORT_GPIOC               ((uint8_t)0x2)
#define GPIO_EXTI_EVEVT_PORT_GPIOD               ((uint8_t)0x3)
#define GPIO_EXTI_EVEVT_PORT_GPIOE               ((uint8_t)0x4)

/*!< Check GPIO port */
#define GPIO_ENENT_PORT_CHECK(port)         (((port) == GPIO_EXTI_EVEVT_PORT_GPIOA) || \
                                              ((port) == GPIO_EXTI_EVEVT_PORT_GPIOB) || \
                                              ((port) == GPIO_EXTI_EVEVT_PORT_GPIOC) || \
                                              ((port) == GPIO_EXTI_EVEVT_PORT_GPIOD) || \
                                              ((port) == GPIO_EXTI_EVEVT_PORT_GPIOE))

/*!< Check GPIO port */
#define GPIO_EXTI_PORT_CHECK(port)          (((port) == GPIO_EXTI_EVEVT_PORT_GPIOA) || \
                                              ((port) == GPIO_EXTI_EVEVT_PORT_GPIOB) || \
                                              ((port) == GPIO_EXTI_EVEVT_PORT_GPIOC) || \
                                              ((port) == GPIO_EXTI_EVEVT_PORT_GPIOD) || \
                                              ((port) == GPIO_EXTI_EVEVT_PORT_GPIOE))


/**
  * @}
  */

/** @defgroup GPIO_event_pin
  * @{
  */

#define GPIO_EXTI_EVENT_PIN0            ((uint8_t)0x00)
#define GPIO_EXTI_EVENT_PIN1            ((uint8_t)0x01)
#define GPIO_EXTI_EVENT_PIN2            ((uint8_t)0x02)
#define GPIO_EXTI_EVENT_PIN3            ((uint8_t)0x03)
#define GPIO_EXTI_EVENT_PIN4            ((uint8_t)0x04)
#define GPIO_EXTI_EVENT_PIN5            ((uint8_t)0x05)
#define GPIO_EXTI_EVENT_PIN6            ((uint8_t)0x06)
#define GPIO_EXTI_EVENT_PIN7            ((uint8_t)0x07)
#define GPIO_EXTI_EVENT_PIN8            ((uint8_t)0x08)
#define GPIO_EXTI_EVENT_PIN9            ((uint8_t)0x09)
#define GPIO_EXTI_EVENT_PIN10           ((uint8_t)0x0A)
#define GPIO_EXTI_EVENT_PIN11           ((uint8_t)0x0B)
#define GPIO_EXTI_EVENT_PIN12           ((uint8_t)0x0C)
#define GPIO_EXTI_EVENT_PIN13           ((uint8_t)0x0D)
#define GPIO_EXTI_EVENT_PIN14           ((uint8_t)0x0E)
#define GPIO_EXTI_EVENT_PIN15           ((uint8_t)0x0F)

/*!< Check GPIO event pin */
#define GPIO_EVENT_PIN_CHECK(pin) (((pin) == GPIO_EXTI_EVENT_PIN0) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN1) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN2) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN3) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN4) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN5) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN6) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN7) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN8) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN9) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN10) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN11) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN12) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN13) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN14) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN15))

/*!< Check GPIO EXTI pin */
#define GPIO_EXTI_PIN_CHECK(pin) (((pin) == GPIO_EXTI_EVENT_PIN0) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN1) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN2) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN3) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN4) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN5) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN6) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN7) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN8) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN9) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN10) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN11) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN12) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN13) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN14) || \
                                   ((pin) == GPIO_EXTI_EVENT_PIN15))

/**
  * @}
  */

/** @defgroup GPIO_remap_define
  * @{
  */

#define GPIO_REMAP_SPI1             ((uint32_t)0x00000001)      /*!< SPI1 AF mapping */
#define GPIO_REMAP_I2C1             ((uint32_t)0x00000002)      /*!< I2C1 AF mapping */
#define GPIO_REMAP_USART1           ((uint32_t)0x00000004)      /*!< USART1 AF mapping */
#define GPIO_REMAP_USART2           ((uint32_t)0x00000008)      /*!< USART2 AF mapping */
#define GPIO_REMAP_PARTIAL_USART3   ((uint32_t)0x00000010)      /*!< USART3 partial AF mapping */
#define GPIO_REMAP_FULL_USART3      ((uint32_t)0x00000030)      /*!< USART3 full AF mapping */
#define GPIO_REMAP_PARTIAL_TIM1     ((uint32_t)0x00000040)      /*!< TIM1 partial AF mapping */
#define GPIO_REMAP_FULL_TIM1        ((uint32_t)0x000000C0)      /*!< TIM1 full AF mapping */
#define GPIO_REMAP_PARTIAL1_TIM2    ((uint32_t)0x00000100)      /*!< TIM2 partial AF mapping 1 */
#define GPIO_REMAP_PARTIAL2_TIM2    ((uint32_t)0x00000200)      /*!< TIM2 partial AF mapping 2 */
#define GPIO_REMAP_FULL_TIM2        ((uint32_t)0x00000300)      /*!< TIM2 full AF mapping */
#define GPIO_REMAP_PARTIAL_TIM3     ((uint32_t)0x00000800)      /*!< TIM3 partial AF mapping */
#define GPIO_REMAP_FULL_TIM3        ((uint32_t)0x00000C00)      /*!< TIM3 full AF mapping */
#define GPIO_REMAP_TIM4             ((uint32_t)0x00001000)      /*!< TIM4 AF mapping */
#define GPIO_REMAP1_CAN1            ((uint32_t)0x00004000)      /*!< CAN1 AF mapping mode 1 */
#define GPIO_REMAP2_CAN1            ((uint32_t)0x00006000)      /*!< CAN1 AF mapping mode 2 */
#define GPIO_REMAP_PD0_1            ((uint32_t)0x00008000)      /*!< PD0 PD1 AF mapping */
#define GPIO_REMAP_SWJ_NJTRST       ((uint32_t)0x01000000)      /*!< Full SWJ without NJRST */
#define GPIO_REMAP_SWJ_JTAG_DISABLE ((uint32_t)0x02000000)      /*!< JTAG-DP Disabled and SW-DP Enabled */
#define GPIO_REMAP_SWJ_DISABLE      ((uint32_t)0x04000000)      /*!< Disable SWD and JTAG */

/*!< Check GPIO remapping */
#define GPIO_REMAP_CHECK(remap)     (((remap) == GPIO_REMAP_SPI1) || \
                                    ((remap) == GPIO_REMAP_I2C1) || \
                                    ((remap) == GPIO_REMAP_USART1) || \
                                    ((remap) == GPIO_REMAP_USART2) || \
                                    ((remap) == GPIO_REMAP_PARTIAL_USART3) || \
                                    ((remap) == GPIO_REMAP_FULL_USART3) || \
                                    ((remap) == GPIO_REMAP_PARTIAL_TIM1) || \
                                    ((remap) == GPIO_REMAP_FULL_TIM1) || \
                                    ((remap) == GPIO_REMAP_PARTIAL1_TIM2) || \
                                    ((remap) == GPIO_REMAP_PARTIAL2_TIM2) || \
                                    ((remap) == GPIO_REMAP_FULL_TIM2) || \
                                    ((remap) == GPIO_REMAP_PARTIAL_TIM3) || \
                                    ((remap) == GPIO_REMAP_FULL_TIM3) || \
                                    ((remap) == GPIO_REMAP_TIM4) || \
                                    ((remap) == GPIO_REMAP1_CAN1) || \
                                    ((remap) == GPIO_REMAP2_CAN1) || \
                                    ((remap) == GPIO_REMAP_PD0_1) || \
                                    ((remap) == GPIO_REMAP_SWJ_NJTRST) || \
                                    ((remap) == GPIO_REMAP_SWJ_JTAG_DISABLE) || \
                                    ((remap) == GPIO_REMAP_SWJ_DISABLE))

/**
  * @}
  */


/*****************************************************************************
 * @functions
*****************************************************************************/


/**@brief       Reset GPIOx registers to default value.
 *
 * @param[in]   PORT: pointer to GPIOx, where x can be A to E .
 *
 * @return      None.
 */
#define __GPIO_DEF_INIT(PORT)               gpio_def_init(PORT)

/**@brief       Clears the selected data port bits.
 *
 * @param[in]   PORT: pointer to GPIOx, where x can be A to E .
 *
 * @param[in]   PIN: specifies the port pin to be set (Support any combination of GPIO_PIN_0 to GPIO_PIN_15).
 *
 * @return      None.
 */
#define __GPIO_PIN_RESET(PORT, PIN)         (PORT->CLRR = (uint16_t)PIN)

/**@brief       Clears the selected data port bits.
 *
 * @param[in]   PORT: pointer to GPIOx, where x can be A to E .
 *
 * @param[in]   PIN: specifies the port pin to be set (Support any combination of GPIO_PIN_0 to GPIO_PIN_15).
 *
 * @return      None.
 */
#define __GPIO_PIN_SET(PORT, PIN)           (PORT->SCR = (uint16_t)PIN)

/**@brief       Writes data to the specified GPIO data port.
 *
 * @param[in]   PORT: pointer to GPIOx, where x can be A to E .
 *
 * @param[in]   DATA: specifies the value to be written to the port output data register.
 *
 * @return      None.
 */
#define __GPIO_DATA_SET(PORT, DATA)         (PORT->DO = DATA)

/**@brief       Reads the specified input port pin.
 *
 * @param[in]   PORT: pointer to GPIOx, where x can be A to E .
 *
 * @return      The input port pin value.
 */
#define __GPIO_INPUT_DATA_GET(PORT)         ((uint16_t)PORT->DI)

/**@brief       Checks the specified input data port bit is set or not.
 *
 * @param[in]   PORT: pointer to GPIOx, where x can be A to E .
 *
 * @param[in]   PIN: specifies the port bits to be check (one of the GPIO_PIN_0 to GPIO_PIN_15).
 *
 * @return      None.
 */
#define __GPIO_INPUT_PIN_GET(PORT, PIN)     ((PORT->DI & (uint16_t)PIN) ? SET : RESET)

/**@brief       Reads the specified output port data.
 *
 * @param[in]   PORT: pointer to GPIOx, where x can be A to E .
 *
 * @return      The output port pin value.
 */
#define __GPIO_OUTPUT_DATA_GET(PORT)        ((uint16_t)PORT->DO)

/**@brief       Checks the specified output data port bit is set or not.
 *
 * @param[in]   PORT: pointer to GPIOx, where x can be A to E .
 *
 * @param[in]   PIN: specifies the port bits to be check (one of the GPIO_PIN_0 to GPIO_PIN_15).
 *
 * @return      None.
 */
#define __GPIO_OUTPUT_PIN_GET(PORT, PIN)    ((PORT->DO & (uint16_t)PIN) ? SET : RESET)

/**@brief       Enable GPIO event output.
 *
 * @param[in]   None.
 *
 * @return      None.
 */
#define __GPIO_EVENT_OUTPUT_ENABLE()        (AFIO->ECR |= AFIO_ECR_EOEN)

/**@brief       Disable GPIO event output.
 *
 * @param[in]   None.
 *
 * @return      None.
 */
#define __GPIO_EVENT_OUTPUT_DISABLE()        (AFIO->ECR &= ~AFIO_ECR_EOEN)

/**@brief       Sets the specified IO as specified mode.
 *
 * @param[in]   ptr_gpio: pointer to GPIOx, where x can be A to E .
 *
 * @param[in]   pin: specifies the port pin to be set (Support any combination of GPIO_PIN_0 to GPIO_PIN_15).
 *
 * @param[in]   mode: Mode selection. This parameter can be one of @ref GPIO_mode_selection.
 *
 * @return      None.
 */
void gpio_mode_config(gpio_reg_t * ptr_gpio, uint16_t pin, uint32_t mode);

/**@brief       Reset GPIOx registers to default value.
 *
 * @param[in]   ptr_gpio: pointer to GPIOx, where x can be A to E .
 *
 * @return      None.
 */
void gpio_def_init(gpio_reg_t * ptr_gpio);

/**@brief       Rest AFIO registers (remap, event control and EXTI configuration) to default value.
 *
 * @param[in]   None.
 *
 * @return      None.
 */
void gpio_af_deinit(void);

/**@brief       Locks GPIO pins configuration registers, The locked registers are GPIOx_CFGL,
 *              GPIOx_CFGH.The sequence is: w1 w0 w1 r0 r1
 *
 * @param[in]   ptr_gpio: pointer to GPIOx, where x can be A to E .
 *
 * @param[in]   pin: specifies the port pin to be set (Support any combination of GPIO_PIN_0 to GPIO_PIN_15).
 *
 * @return      None.
 */
void gpio_pin_lock_set(gpio_reg_t * ptr_gpio, uint16_t pin);

/**@brief       Config GPIO event port and pin.
 *
 * @param[in]   event_port: specifies GPIO. It can be any combination of the following values: @ref GPIO_event_port.
 *
 * @param[in]   event_pin: specifies pin. It can be any combination of the following values: @ref GPIO_event_pin.
 *
 * @return      None.
 */
void gpio_event_config(uint8_t event_port, uint8_t event_pin);

/**@brief       Config GPIO remap pin.
 *
 * @param[in]   remap_peri: specifies peripherals. It can be any combination of the following values: @ref GPIO_remap_define.
 *
 * @param[in]   flag: ENABLE or DISABLE.
 *
 * @return      None.
 */
void gpio_pin_remap_config(uint32_t remap_peri, enable_state_t flag);

/**@brief       Config GPIO exti port and pin.
 *
 * @param[in]   exti_port: specifies GPIO. It can be any combination of the following values: @ref GPIO_event_port.
 *
 * @param[in]   exti_pin: specifies pin. It can be any combination of the following values: @ref GPIO_event_pin.
 *
 * @return      None.
 */
void gpio_exti_pin_config(uint8_t exti_port, uint8_t exti_pin);


/*@}*/ /* end of group GPIO */

#ifdef __cplusplus
}
#endif


#endif


/*** (C) COPYRIGHT 2021 Software Development. ***/



