/***************************************************************************//**
 * @file        cs32f10x_misc.h
 * @version     V2.0.1
 * @author      Software Development
 * @brief       This file contains the definition of misc function parameters and function interface declaration.
 *              
 * @copyright   Copyright (C) Software Development. All rights reserved.
****************************************************************************/

#ifndef __CS32F10X_MISC_H__
#define __CS32F10X_MISC_H__

#ifdef __cplusplus
     extern "C" {
#endif


/*****************************************************************************
 * @includes 
*****************************************************************************/

#include "cs32f10x.h"


/** @addtogroup MISC
  Configuration of the M3 Processor and Core Peripherals:
###Features: 
    - interrupt numbers
    - registers and bit fields
    - peripheral base address
    - peripheral ID
    - Peripheral definitions
  @{
*/


/** 
  * @brief nvic struct definition  
  */
typedef struct
{
    uint8_t nvic_irqchannel;                /*!< The IRQ channel to be ENABLE or DISABLE.
                                                 This parameter can be one of the following values: @ref IRQn_Type */
    
    uint8_t nvic_irq_pre_priority;          /*!< The pre-emption priority. This parameter can be a value
                                                 between 0 and 15.Please @ref NVIC_Priority_Table */
    
    uint8_t nvic_irq_sub_priority;          /*!< The subpriority level.This parameter can be a value
                                                 between 0 and 15.Please @ref NVIC_Priority_Table */
    
    enable_state_t nvic_irq_enable;         /*!< ENABLE or DISABLE */
    
}nvic_init_t;

/*****************************************************************************
 * @definitions 
*****************************************************************************/

/** @defgroup NVIC_Priority_Table
  * @{
  */
/*
============================================================================================================================
    NVIC_PriorityGroup   | nvic_irq_pre_priority             | nvic_irq_sub_priority       | Description
  ============================================================================================================================
   NVIC_PriorityGroup_0  |                0                  |            0-15             |   0 bits for pre-emption priority
                         |                                   |                             |   4 bits for subpriority
  ----------------------------------------------------------------------------------------------------------------------------
   NVIC_PriorityGroup_1  |                0-1                |            0-7              |   1 bits for pre-emption priority
                         |                                   |                             |   3 bits for subpriority
  ----------------------------------------------------------------------------------------------------------------------------    
   NVIC_PriorityGroup_2  |                0-3                |            0-3              |   2 bits for pre-emption priority
                         |                                   |                             |   2 bits for subpriority
  ----------------------------------------------------------------------------------------------------------------------------    
   NVIC_PriorityGroup_3  |                0-7                |            0-1              |   3 bits for pre-emption priority
                         |                                   |                             |   1 bits for subpriority
  ----------------------------------------------------------------------------------------------------------------------------    
   NVIC_PriorityGroup_4  |                0-15               |            0                |   4 bits for pre-emption priority
                         |                                   |                             |   0 bits for subpriority                       
  ============================================================================================================================
*/

/**
  * @}
  */
  

/** @defgroup NVIC_vector_mapping
  * @{
  */
  
#define NVIC_VECT_MAPPING_RAM           ((uint32_t)0x20000000)
#define NVIC_VECT_MAPPING_FLASH         ((uint32_t)0x08000000)

#define NVIC_VECT_MAPPING_CHECK(vect)   (((vect) == NVIC_VECT_MAPPING_RAM) || ((vect) == NVIC_VECT_MAPPING_FLASH))
/**
  * @}
  */

/** @defgroup NVIC_system_low_power
  * @{
  */

#define NVIC_LP_SEVONPEND            ((uint8_t)0x10)
#define NVIC_LP_SLEEPDEEP            ((uint8_t)0x04)
#define NVIC_LP_SLEEPONEXIT          ((uint8_t)0x02)

#define NVIC_LP_CHECK(power)          (((power) == NVIC_LP_SEVONPEND) || \
                                        ((power) == NVIC_LP_SLEEPDEEP) || \
                                        ((power) == NVIC_LP_SLEEPONEXIT))
/**
  * @}
  */

/** @defgroup Preemption_Priority_Group 
  * @{
  */

#define NVIC_PriorityGroup_0         ((uint32_t)0x700) /*!< 0 bits for pre-emption priority
                                                            4 bits for subpriority */
#define NVIC_PriorityGroup_1         ((uint32_t)0x600) /*!< 1 bits for pre-emption priority
                                                            3 bits for subpriority */
#define NVIC_PriorityGroup_2         ((uint32_t)0x500) /*!< 2 bits for pre-emption priority
                                                            2 bits for subpriority */
#define NVIC_PriorityGroup_3         ((uint32_t)0x400) /*!< 3 bits for pre-emption priority
                                                            1 bits for subpriority */
#define NVIC_PriorityGroup_4         ((uint32_t)0x300) /*!< 4 bits for pre-emption priority
                                                            0 bits for subpriority */

#define NVIC_PRIORITY_GROUP_CHECK(GROUP) (((GROUP) == NVIC_PriorityGroup_0) || \
                                        ((GROUP) == NVIC_PriorityGroup_1) || \
                                        ((GROUP) == NVIC_PriorityGroup_2) || \
                                        ((GROUP) == NVIC_PriorityGroup_3) || \
                                        ((GROUP) == NVIC_PriorityGroup_4))

#define NVIC_PREEMPTION_PRIORITY_CHECK(PRIORITY)  ((PRIORITY) < 0x10)

#define NVIC_SUB_PRIORITY_CHECK(PRIORITY)  ((PRIORITY) < 0x10)

#define NVIC_OFFSET_CHECK(OFFSET)  ((OFFSET) < 0x000FFFFF)

/**
  * @}
  */

/** @defgroup SysTick_clock_source 
  * @{
  */

#define SysTick_CLKSource_HCLK_Div8    ((uint32_t)0xFFFFFFFB)
#define SysTick_CLKSource_HCLK         ((uint32_t)0x00000004)
#define SYSTICK_CLK_SOURCE_CHECK(SOURCE) (((SOURCE) == SysTick_CLKSource_HCLK) || \
                                          ((SOURCE) == SysTick_CLKSource_HCLK_Div8))
/**
  * @}
  */


/**@brief       Configures the priority grouping.
 *
 * @param[in]   priority_group: It can be one of the following values: @ref Preemption_Priority_Group
 *
 * @return      None.
 */
void nvic_priority_group_config(uint32_t priority_group);

/**@brief       Initializes the NVIC peripheral according to the specified parameters in the nvic_struct.
 *
 * @param[in]   nvic_struct: pointer to a nvic_init_t structure.
 *
 * @return      None.
 */
void nvic_init(nvic_init_t* nvic_struct);

/**@brief       Sets the vector table location and Offset.
 *
 * @param[in]   vec_tab: vector table is in RAM or FLASH memory.This parameter can be one of the following values:
 *              @ref NVIC_vector_mapping
 *
 * @param[in]   offset: Vector Table base offset field. This value must be a multiple of 0x200.
 *
 * @return      None.
 */
void nvic_vector_table_set(uint32_t vec_tab, uint32_t offset);

/**@brief       Selects the condition for the system to enter low power mode.
 *
 * @param[in]   low_power_mode: Specifies the new mode for the system to enter low power mode.
 *              This parameter can be one of the following values: @ref NVIC_system_low_power
 *
 * @param[in]   status: ENABLE or DISABLE the mode.
 *
 * @return      None.
 */
void nvic_sys_lowpwr_config(uint8_t low_power_mode, enable_state_t status);

/**@brief       Configures the SysTick clock source.
 *
 * @param[in]   systick_clk_source: specifies the SysTick clock source.
 *              This parameter can be one of the following values: @ref SysTick_clock_source
 *
 * @return      None.
 */
void systick_clk_src_config(uint32_t systick_clk_source);


/*@}*/ /* end of group MISC */


#ifdef __cplusplus
}
#endif

#endif

/*** (C) COPYRIGHT 2021 Software Development . ***/
