/***************************************************************************//**
 * @file        cs32f10x_rcu.h
 * @version     V2.0.1
 * @author      Software Development
 * @brief       This file contains all the functions prototypes for the RCU firmware library.
 *              
 * @copyright   Copyright (C) Software Development. All rights reserved.
 ****************************************************************************/

#ifndef __CS32F10X_RCU_H__
#define __CS32F10X_RCU_H__

#ifdef __cplusplus
     extern "C" {
#endif


/*****************************************************************************
 * @includes 
*****************************************************************************/

#include "cs32f10x.h"


/** @addtogroup RCU
  Configuration of the M3 Processor and Core Peripherals:
###Features: 
    - interrupt numbers
    - registers and bit fields
    - peripheral base address
    - peripheral ID
    - Peripheral definitions
  @{
*/


/** 
  * @brief RCU frequency structure.
  */ 
typedef struct
{
    uint32_t sysclk_freq;       /*!< System clock frequency */
    uint32_t hclk_freq;         /*!< HCLK frequency */
    uint32_t pclk1_freq;        /*!< PCLK1 frequency */
    uint32_t pclk2_freq;        /*!< PCLK2 frequency */
    uint32_t adc_clk_freq;    /*!< ADC clock frequency */
}rcu_clock_t;


/*****************************************************************************
 * @definitions 
*****************************************************************************/

/** @defgroup RCU_hxt_state
  * @{
  */

#define RCU_HXT_ENABLE      RCU_CTR_HXTEN            /*!< Enable the RCU */
#define RCU_HXT_DISABLE     ((uint32_t)0x00000000)  /*!< Disable the RCU */
#define RCU_HXT_BYPASS      ((uint32_t)0x00040000)  /*!< RCU external osi bypass */

/*!< Check RCU HXT status */
#define RCU_HXT_STAT_CHECK(value)       (((value) == RCU_HXT_ENABLE) || \
                                        ((value) == RCU_HXT_DISABLE) || \
                                        ((value) == RCU_HXT_BYPASS))
/**
  * @}
  */

/*!< Check RCU HRC trim value */
#define RCU_HRC_CALI_VALUE_CHECK(value)     ((value) <= 0x1F)

/** @defgroup RCU_pll_src
  * @{
  */

#define RCU_PLL_SOURCE_HRC_DIV2         ((uint32_t)0x00000000)
#define RCU_PLL_SOURCE_HXT_DIV1         ((uint32_t)0x00010000)
#define RCU_PLL_SOURCE_HXT_DIV2         ((uint32_t)0x00030000)

/*!< Check RCU pll source config */
#define RCU_PLL_SOURCE_CHECK(source)    (((source) == RCU_PLL_SOURCE_HRC_DIV2) || ((source) == RCU_PLL_SOURCE_HXT_DIV1) || \
                                         ((source) == RCU_PLL_SOURCE_HXT_DIV2))
/**
  * @}
  */

/** @defgroup RCU_pll_multiple
  * @{
  */
  
#define RCU_PLL_MULTI_2                 ((uint32_t)0x00000000)
#define RCU_PLL_MULTI_3                 ((uint32_t)0x00040000)
#define RCU_PLL_MULTI_4                 ((uint32_t)0x00080000)
#define RCU_PLL_MULTI_5                 ((uint32_t)0x000C0000)
#define RCU_PLL_MULTI_6                 ((uint32_t)0x00100000)
#define RCU_PLL_MULTI_7                 ((uint32_t)0x00140000)
#define RCU_PLL_MULTI_8                 ((uint32_t)0x00180000)
#define RCU_PLL_MULTI_9                 ((uint32_t)0x001C0000)
#define RCU_PLL_MULTI_10                ((uint32_t)0x00200000)
#define RCU_PLL_MULTI_11                ((uint32_t)0x00240000)
#define RCU_PLL_MULTI_12                ((uint32_t)0x00280000)
#define RCU_PLL_MULTI_13                ((uint32_t)0x002C0000)
#define RCU_PLL_MULTI_14                ((uint32_t)0x00300000)
#define RCU_PLL_MULTI_15                ((uint32_t)0x00340000)
#define RCU_PLL_MULTI_16                ((uint32_t)0x00380000)

/*!< Check RCU pll multiplication */
#define RCU_PLL_MULTI_CHECK(value)      (((value) == RCU_PLL_MULTI_2)   || ((value) == RCU_PLL_MULTI_3)  || \
                                         ((value) == RCU_PLL_MULTI_4)   || ((value) == RCU_PLL_MULTI_5)  || \
                                         ((value) == RCU_PLL_MULTI_6)   || ((value) == RCU_PLL_MULTI_7)  || \
                                         ((value) == RCU_PLL_MULTI_8)   || ((value) == RCU_PLL_MULTI_9)  || \
                                         ((value) == RCU_PLL_MULTI_10)  || ((value) == RCU_PLL_MULTI_11) || \
                                         ((value) == RCU_PLL_MULTI_12)  || ((value) == RCU_PLL_MULTI_13) || \
                                         ((value) == RCU_PLL_MULTI_14)  || ((value) == RCU_PLL_MULTI_15) || \
                                         ((value) == RCU_PLL_MULTI_16))
/**
  * @}
  */

/** @defgroup RCU_clk_out
  * @{
  */

#define RCU_CLKOUT_SEL_NO_CLK           ((uint32_t)0x00000000)
#define RCU_CLKOUT_SEL_SYSCLK           ((uint32_t)0x04000000)
#define RCU_CLKOUT_SEL_HRCCLK           ((uint32_t)0x05000000)
#define RCU_CLKOUT_SEL_HXTCLK           ((uint32_t)0x06000000)
#define RCU_CLKOUT_SEL_PLLCLK_DIV2      ((uint32_t)0x07000000)

/*!< Check rcu clock output source */
#define RCU_CLKOUT_SRC_CHECK(source)    (((source) == RCU_CLKOUT_SEL_NO_CLK) || ((source) == RCU_CLKOUT_SEL_SYSCLK) || \
                                         ((source) == RCU_CLKOUT_SEL_HRCCLK) || ((source) == RCU_CLKOUT_SEL_HXTCLK) || \
                                         ((source) == RCU_CLKOUT_SEL_PLLCLK_DIV2))
/**
  * @}
  */

/** @defgroup RCU_sysclk_source
  * @{
  */

#define RCU_SYSCLK_SEL_HRC              ((uint32_t)0x00000000)      /*!< RCU config system clock sources is HRC */
#define RCU_SYSCLK_SEL_HXT              ((uint32_t)0x00000001)      /*!< RCU config system clock sources is HXT */
#define RCU_SYSCLK_SEL_PLL              ((uint32_t)0x00000002)      /*!< RCU config system clock sources is PLL */

/*!< Check RCU sysclk source */
#define RCU_SYSCLK_SEL_CHECK(value)         (((value) == RCU_SYSCLK_SEL_HRC)   || ((value) == RCU_SYSCLK_SEL_HXT) || \
                                             ((value) == RCU_SYSCLK_SEL_PLL))
/**
  * @}
  */

/** @defgroup RCU_pclk_source
  * @{
  */

#define RCU_HCLK_DIV_1                  ((uint32_t)0x00000000)
#define RCU_HCLK_DIV_2                  ((uint32_t)0x00000400)
#define RCU_HCLK_DIV_4                  ((uint32_t)0x00000500)
#define RCU_HCLK_DIV_8                  ((uint32_t)0x00000600)
#define RCU_HCLK_DIV_16                 ((uint32_t)0x00000700)

/*!< Check RCU PCLK source */
#define RCU_PCLK_SEL_CHECK(source)      (((source) == RCU_HCLK_DIV_1)   || ((source) == RCU_HCLK_DIV_2) || \
                                        ((source) == RCU_HCLK_DIV_4) || ((source) == RCU_HCLK_DIV_8) || \
                                        ((source) == RCU_HCLK_DIV_16))
/**
  * @}
  */

/** @defgroup RCU_hclk_source
  * @{
  */
  
#define RCU_SYSCLK_DIV1                 ((uint32_t)0x00000000)
#define RCU_SYSCLK_DIV2                 ((uint32_t)0x00000080)
#define RCU_SYSCLK_DIV4                 ((uint32_t)0x00000090)
#define RCU_SYSCLK_DIV8                 ((uint32_t)0x000000A0)
#define RCU_SYSCLK_DIV16                ((uint32_t)0x000000B0)
#define RCU_SYSCLK_DIV64                ((uint32_t)0x000000C0)
#define RCU_SYSCLK_DIV128               ((uint32_t)0x000000D0)
#define RCU_SYSCLK_DIV256               ((uint32_t)0x000000E0)
#define RCU_SYSCLK_DIV512               ((uint32_t)0x000000F0)

/*!< Check RCU HCLK source */
#define RCU_HCLK_SEL_CHECK(source)      (((source) == RCU_SYSCLK_DIV1) || ((source) == RCU_SYSCLK_DIV2) || \
                                        ((source) == RCU_SYSCLK_DIV4) || ((source) == RCU_SYSCLK_DIV8) || \
                                        ((source) == RCU_SYSCLK_DIV16) || ((source) == RCU_SYSCLK_DIV64) || \
                                        ((source) == RCU_SYSCLK_DIV128) || ((source) == RCU_SYSCLK_DIV256) || \
                                        ((source) == RCU_SYSCLK_DIV512))
/**
  * @}
  */

/** @defgroup RCU_adcclk_source
  * @{
  */

#define RCU_ADCCLK_SEL_PCLK2_DIV2       ((uint32_t)0x00000000)
#define RCU_ADCCLK_SEL_PCLK2_DIV4       ((uint32_t)0x00004000)
#define RCU_ADCCLK_SEL_PCLK2_DIV6       ((uint32_t)0x00008000)
#define RCU_ADCCLK_SEL_PCLK2_DIV8       ((uint32_t)0x0000C000)

/*!< Check RCU ADC clock source */
#define RCU_ADCCLK_SEL_CHECK(source)    (((source) == RCU_ADCCLK_SEL_PCLK2_DIV2) || ((source) == RCU_ADCCLK_SEL_PCLK2_DIV4) || \
                                        ((source) == RCU_ADCCLK_SEL_PCLK2_DIV6) || ((source) == RCU_ADCCLK_SEL_PCLK2_DIV8))
/**
  * @}
  */

/** @defgroup RCU_usbclk_source
  * @{
  */

#define RCU_USBCLK_SEL_PLLCLK_Div1_5    ((uint32_t)0x00000000)      /*!< PLLCLK divided 1.5 as USB clock */
#define RCU_USBCLK_SEL_PLLCLK_Div1      ((uint32_t)0x00400000)      /*!< PLLCLK  as USB clock */

/*!< Check RCU USB clock source */
#define RCU_USBCLK_SEL_CHECK(source)    (((source) == RCU_USBCLK_SEL_PLLCLK_Div1_5) || ((source) == RCU_USBCLK_SEL_PLLCLK_Div1))
/**
  * @}
  */

/** @defgroup RCU_rtcclk_source
  * @{
  */

#define RCU_RTCCLK_SEL_LXT              ((uint32_t)0x00000100)
#define RCU_RTCCLK_SEL_LRC              ((uint32_t)0x00000200)
#define RCU_RTCCLK_SEL_HXT_DIV128       ((uint32_t)0x00000300)

/*!< Check RCU RTC clock source */
#define RCU_RTCCLK_SEL_CHECK(source)    (((source) == RCU_RTCCLK_SEL_LXT) || ((source) == RCU_RTCCLK_SEL_LRC) || \
                                            ((source) == RCU_RTCCLK_SEL_HXT_DIV128))
/**
  * @}
  */

/** @defgroup RCU_interrupt_enable
  * @{
  */

#define RCU_INTR_LRC_STAB               RCU_INTR_LRCSTABIE            /*!< RCU LRC stabilization interrupt */
#define RCU_INTR_LXT_STAB               RCU_INTR_LXTSTABIE            /*!< RCU LXT stabilization interrupt */
#define RCU_INTR_HRC_STAB               RCU_INTR_HRCSTABIE            /*!< RCU HRC stabilization interrupt */
#define RCU_INTR_HXT_STAB               RCU_INTR_HXTSTABIE            /*!< RCU HRC stabilization interrupt */
#define RCU_INTR_PLL_STAB               RCU_INTR_PLLSTABIE            /*!< RCU PLL stabilization interrupt */
/**
  * @}
  */

/** @defgroup RCU_interrupt_status_flag
  * @{
  */

#define RCU_INTR_FLAG_LRC               RCU_INTR_LRCSTABIF             /*!< RCU LRC interrupt flag */
#define RCU_INTR_FLAG_LXT               RCU_INTR_LXTSTABIF             /*!< RCU LXT interrupt flag */
#define RCU_INTR_FLAG_HRC               RCU_INTR_HRCSTABIF             /*!< RCU HRC interrupt flag */
#define RCU_INTR_FLAG_HXT               RCU_INTR_HXTSTABIF             /*!< RCU HXT interrupt flag */
#define RCU_INTR_FLAG_PLL               RCU_INTR_PLLSTABIF             /*!< RCU PLL interrupt flag */
#define RCU_INTR_FLAG_CLK_SECURITY      RCU_INTR_CKFAILIF              /*!< RCU clock security interrupt flag */
/**
  * @}
  */

/** @defgroup RCU_interrupt_clear_flag
  * @{
  */

#define RCU_INTR_FLAG_CLR_LRC           RCU_INTR_LRCSTABIC             /*!< RCU LRC interrupt flag clear */
#define RCU_INTR_FLAG_CLR_LXT           RCU_INTR_LXTSTABIC             /*!< RCU LXT interrupt flag clear */
#define RCU_INTR_FLAG_CLR_HRC           RCU_INTR_HRCSTABIC             /*!< RCU HRC interrupt flag clear */
#define RCU_INTR_FLAG_CLR_HXT           RCU_INTR_HXTSTABIC             /*!< RCU HXT interrupt flag clear */
#define RCU_INTR_FLAG_CLR_PLL           RCU_INTR_PLLSTABIC             /*!< RCU PLL interrupt flag clear */
#define RCU_INTR_FLAG_CLR_CLK_SECURITY  RCU_INTR_CKFAILIC              /*!< RCU clock security flag clear */
/**
  * @}
  */

/** @defgroup RCU_APB2_reset
  * @{
  */

#define RCU_RST_AFIO                    RCU_APB2RST_AFIORST        /*!< Multi-function reset */
#define RCU_RST_GPIOA                   RCU_APB2RST_PARST        /*!< GPIOA reset */
#define RCU_RST_GPIOB                   RCU_APB2RST_PBRST        /*!< GPIOB reset */
#define RCU_RST_GPIOC                   RCU_APB2RST_PCRST        /*!< GPIOC reset */
#define RCU_RST_GPIOD                   RCU_APB2RST_PDRST        /*!< GPIOD reset */
#define RCU_RST_GPIOE                   RCU_APB2RST_PERST        /*!< GPIOE reset */
#define RCU_RST_ADC1                    RCC_APB2RST_ADC1RST        /*!< ADC1 reset */
#define RCU_RST_ADC2                    RCC_APB2RST_ADC2RST        /*!< ADC2 reset */
#define RCU_RST_TIM1                    RCC_APB2RST_TIM1RST        /*!< TIM1 reset */
#define RCU_RST_SPI1                    RCC_APB2RST_SPI1RST        /*!< SPI1 reset */
#define RCU_RST_USART1                  RCC_APB2RST_USART1RST      /*!< USART1 reset */
/**
  * @}
  */

/** @defgroup RCU_APB1_reset
  * @{
  */

#define RCU_RST_TIM2                    RCU_APB1RST_TIM2RST        /*!< TIM2 reset */
#define RCU_RST_TIM3                    RCU_APB1RST_TIM3RST        /*!< TIM3 reset */
#define RCU_RST_TIM4                    RCU_APB1RST_TIM4RST        /*!< TIM4 reset */
#define RCU_RST_WWDT                    RCU_APB1RST_WWDTRST        /*!< WWDT reset */
#define RCU_RST_SPI2                    RCU_APB1RST_SPI2RST        /*!< SPI2 reset */
#define RCU_RST_USART2                  RCU_APB1RST_USART2RST      /*!< USART2 reset */
#define RCU_RST_USART3                  RCU_APB1RST_USART3RST      /*!< USART3 reset */
#define RCU_RST_I2C1                    RCU_APB1RST_I2C1RST        /*!< I2C2 reset */
#define RCU_RST_I2C2                    RCU_APB1RST_I2C2RST        /*!< I2C2 reset */
#define RCU_RST_USB                     RCU_APB1RST_USBRST         /*!< USB reset */
#define RCU_RST_CAN1                    RCU_APB1RST_CAN1RST        /*!< CAN1 reset */
#define RCU_RST_BAKUP                   RCU_APB1RST_BKPRST         /*!< BKP reset */
#define RCU_RST_PMU                     RCU_APB1RST_PMURST         /*!< PMU reset */
/**
  * @}
  */

#define RCU_RST_BKPD                    RCU_VBDC_VBTRST              /*!< Backup domain reset */

/** @defgroup RCU_AHB_clock_enable
  * @{
  */

#define RCU_AHB_PERI_DMA                RCU_AHBEN_DMA1EN
#define RCU_AHB_PERI_SRAM               RCU_AHBEN_RMCEN
#define RCU_AHB_PERI_FMC                RCU_AHBEN_FMCEN
#define RCU_AHB_PERI_CRC                RCU_AHBEN_CRCEN
/**
  * @}
  */

/** @defgroup RCU_APB1_clock_enable
  * @{
  */

#define RCU_APB1_PERI_TIM2              RCU_APB1EN_TIM2EN
#define RCU_APB1_PERI_TIM3              RCU_APB1EN_TIM3EN
#define RCU_APB1_PERI_TIM4              RCU_APB1EN_TIM4EN
#define RCU_APB1_PERI_WWDT              RCU_APB1EN_WWDTEN
#define RCU_APB1_PERI_SPI2              RCU_APB1EN_SPI2EN
#define RCU_APB1_PERI_USART2            RCU_APB1EN_USART2EN
#define RCU_APB1_PERI_USART3            RCU_APB1EN_USART3EN
#define RCU_APB1_PERI_I2C1              RCU_APB1EN_I2C1EN
#define RCU_APB1_PERI_I2C2              RCU_APB1EN_I2C2EN
#define RCU_APB1_PERI_USB               RCU_APB1EN_USBEN
#define RCU_APB1_PERI_CAN1              RCU_APB1EN_CAN1EN
#define RCU_APB1_PERI_BAKUP             RCC_APB1EN_BKPEN
#define RCU_APB1_PERI_PMU               RCC_APB1EN_PMUEN
/**
  * @}
  */


/** @defgroup RCU_APB2_clock_enable
  * @{
  */

#define RCU_APB2_PERI_AFIO              RCU_APB2EN_AFIOEN
#define RCU_APB2_PERI_GPIOA             RCU_APB2EN_PAEN
#define RCU_APB2_PERI_GPIOB             RCU_APB2EN_PBEN
#define RCU_APB2_PERI_GPIOC             RCU_APB2EN_PCEN
#define RCU_APB2_PERI_GPIOD             RCU_APB2EN_PDEN
#define RCU_APB2_PERI_GPIOE             RCU_APB2EN_PEEN
#define RCU_APB2_PERI_ADC1              RCU_APB2EN_ADC1EN
#define RCU_APB2_PERI_ADC2              RCU_APB2EN_ADC2EN
#define RCU_APB2_PERI_TIM1              RCU_APB2EN_TIM1EN
#define RCU_APB2_PERI_SPI1              RCU_APB2EN_SPI1EN
#define RCU_APB2_PERI_USART1            RCU_APB2EN_USART1EN
/**
  * @}
  */

/** @defgroup RCU_clk_ready_reset_flag
  * bit[4:0]: Indicates the position of the flag bit in the register.
  * bit[6:5]: Indicates which register. 
  *           1: RCU->CTR
  *           2: RCU->VBDC
  *           3: RCU->STS
  * @{
  */

#define RCU_FLAG_PLL_STABLE             ((uint8_t)0x39)         /*!< PLL stabilization flag */
#define RCU_FLAG_HXT_STABLE             ((uint8_t)0x31)         /*!< HXT stabilization flag */
#define RCU_FLAG_HRC_STABLE             ((uint8_t)0x21)         /*!< HRC stabilization flag */
#define RCU_FLAG_LXT_STABLE             ((uint8_t)0x41)         /*!< LXT stabilization flag */
#define RCU_FLAG_LRC_STABLE             ((uint8_t)0x61)         /*!< LRC stabilization flag */
#define RCU_FLAG_LOW_PWR_RST            ((uint8_t)0x7F)         /*!< Low power reset flag */
#define RCU_FLAG_WWDT_RESET             ((uint8_t)0x7E)         /*!< Window watchdog reset flag */
#define RCU_FLAG_FWDT_RESET             ((uint8_t)0x7D)         /*!< Free watchdog reset flag */
#define RCU_FLAG_SOFT_RESET             ((uint8_t)0x7C)         /*!< Software reset flag */
#define RCU_FLAG_POWUP_RESET            ((uint8_t)0x7B)         /*!< Power up reset flag */
#define RCU_FLAG_PIN_RESET              ((uint8_t)0x7A)         /*!< NRST pin reset flag */

/*!< Check RCU clock flag and reset flag */
#define RCU_FLAG_CLK_READY_RST_CHECK(flag)  (((flag) == RCU_FLAG_PLL_STABLE) || ((flag) == RCU_FLAG_HXT_STABLE) || \
                                            ((flag) == RCU_FLAG_HRC_STABLE) || ((flag) == RCU_FLAG_LXT_STABLE)   || \
                                            ((flag) == RCU_FLAG_LRC_STABLE) || ((flag) == RCU_FLAG_LOW_PWR_RST)  || \
                                            ((flag) == RCU_FLAG_WWDT_RESET) || ((flag) == RCU_FLAG_FWDT_RESET)  || \
                                            ((flag) == RCU_FLAG_SOFT_RESET) || ((flag) == RCU_FLAG_POWUP_RESET)  || \
                                            ((flag) == RCU_FLAG_PIN_RESET))


/**
  * @}
  */


/**
  * @brief RCU function register definition
  */
#define RCU_ENABLE_REG_PLL_CLK          RCU->CTR
#define RCU_ENABLE_REG_CLK_SECURITY     RCU->CTR
#define RCU_ENABLE_REG_HXT_BYPASS       RCU->CTR
#define RCU_ENABLE_REG_HXT_CLK          RCU->CTR
#define RCU_ENABLE_REG_HRC_CLK          RCU->CTR

#define RCU_ENABLE_REG_LXT_CLK          RCU->VBDC
#define RCU_ENABLE_REG_LXT_BYPASS       RCU->VBDC


/**
  * @brief RCU function bit definition
  */
#define RCU_ENABLE_BIT_PLL_CLK          RCU_CTR_PLLEN
#define RCU_ENABLE_BIT_CLK_SECURITY     RCU_CTR_HXTME
#define RCU_ENABLE_BIT_HXT_BYPASS       RCU_CTR_HXTBPS
#define RCU_ENABLE_BIT_HXT_CLK          RCU_CTR_HXTEN
#define RCU_ENABLE_BIT_HRC_CLK          RCU_CTR_HRCEN

#define RCU_ENABLE_BIT_LXT_CLK          RCU_VBDC_LXTEN
#define RCU_ENABLE_BIT_LXT_BYPASS       RCU_VBDC_LXTBYP




/**@brief       Resets the RCU clock configuration to the default reset state.
 *              The default reset state including HRC ON and used as system clock source HRC, HXT and PLL OFF AHB, APB 
 *              prescaler is set to 1, CLKOUT is off.  All interrupts is disabled, but it doesn't modify the
 *              configuration of the peripheral clocks LRC, LXT and RTC clocks.
 *
 * @param       None.
 *
 * @return      None.
 */
#define __RCU_DEF_INIT()                rcu_def_init()


/**@brief  Enables the specified RCU function.           
 * 
 * @param  FUNC: specifies the RCU function, it can be one of the following values:
 *         PLL_CLK (PLL clock)
 *         HXT_CLK (HXT clock)
 *         HXT_BYPASS (HXT bypass)     
 *         CLK_SECURITY (clock security system)  
 *         HRC_CLK (HRC clock)  
 *         LXT_CLK (LXT clock)
 *         LXT_BYPASS (LXT bypass)
 *
 * @return None.
 */
#define __RCU_FUNC_ENABLE(FUNC)         ((RCU_ENABLE_REG_##FUNC) |= (RCU_ENABLE_BIT_##FUNC))

/**@brief  Disable the specified RCU function.
 * 
 * @param  FUNC: specifies the RCU function, it can be one of the following values:
 *         PLL_CLK (PLL clock)
 *         HXT_CLK (HXT clock)
 *         HXT_BYPASS (HXT bypass)     
 *         CLK_SECURITY (clock security system)  
 *         HRC_CLK (HRC clock)  
 *         LXT_CLK (LXT clock)
 *         LXT_BYPASS (LXT bypass)
 *
 * @return None.
 */
#define __RCU_FUNC_DISABLE(FUNC)         ((RCU_ENABLE_REG_##FUNC) &= ~(RCU_ENABLE_BIT_##FUNC))


/**@brief       Enables the specified RCU interrupts.
 *
 * @param[in]   INTR: specifies the RCU interrupt sources to be enabled. It can be any combination of the following values:
 *              @arg RCU_INTR_LRC_STAB (LRC stabilization interrupt)
 *              @arg RCU_INTR_LXT_STAB (LXT stabilization interrupt)
 *              @arg RCU_INTR_HRC_STAB (HRC stabilization interrupt)
 *              @arg RCU_INTR_HXT_STAB (HXT stabilization interrupt)
 *              @arg RCU_INTR_PLL_STAB (PLL stabilization interrupt)
 *
 * @return      None.
 */
#define __RCU_INTR_ENABLE(INT)          (RCU->INTR |= (INT))

/**@brief       Disables the specified RCU interrupts.
 *
 * @param[in]   INTR: specifies the RCU interrupt sources to be disabled. It can be any combination of the following values:
 *              @arg RCU_INTR_LRC_STAB (LRC stabilization interrupt)
 *              @arg RCU_INTR_LXT_STAB (LXT stabilization interrupt)
 *              @arg RCU_INTR_HRC_STAB (HRC stabilization interrupt)
 *              @arg RCU_INTR_HXT_STAB (HXT stabilization interrupt)
 *              @arg RCU_INTR_PLL_STAB (PLL stabilization interrupt)
 *
 * @return      None.
 */
#define __RCU_INTR_DISABLE(INT)         (RCU->INTR &= ~(INT))

/**@brief       Checks whether the specified RCU stabilization flag is set or not.
 *
 * @param[in]   FLAG: specifies the RCU interrupt source to check. It can be any combination of the following values:
 *              @arg RCU_INTR_FLAG_LRC (LRC stabilization interrupt flag)
 *              @arg RCU_INTR_FLAG_LXT (LXT stabilization interrupt flag)
 *              @arg RCU_INTR_FLAG_HRC (HRC stabilization interrupt flag)
 *              @arg RCU_INTR_FLAG_HXT (HXT stabilization interrupt flag)
 *              @arg RCU_INTR_FLAG_PLL (PLL stabilization interrupt flag)
 *              @arg RCU_INTR_FLAG_CLK_SECURITY (RCU clock security interrupt flag)
 *
 * @return      The new state of flag (SET or RESET).
 */
#define __RCU_INTR_STATUS_GET(FLAG)         ((RCU->INTR & (FLAG)) ? SET : RESET)

/**@brief       Checks whether the specified RCU stabilization flag is set or not.
 *
 * @param[in]   FLAG: specifies the RCU interrupt source to check. It can be any combination of the following values:
 *              @arg RCU_INTR_FLAG_CLR_LRC (RCU LRC interrupt flag clear)
 *              @arg RCU_INTR_FLAG_CLR_LXT (RCU LXT interrupt flag clear)
 *              @arg RCU_INTR_FLAG_CLR_HRC (RCU HRC interrupt flag clear)
 *              @arg RCU_INTR_FLAG_CLR_HXT (RCU HXT interrupt flag clear)
 *              @arg RCU_INTR_FLAG_CLR_PLL (RCU PLL interrupt flag clear)
 *              @arg RCU_INTR_FLAG_CLR_CLK_SECURITY (RCU clock security flag clear)
 *
 * @return      None.
 */
#define __RCU_INTR_FLAG_CLEAR(FLAG)         (RCU->INTR |= (FLAG))

/**@brief       Enables the specified apb2 peripheral reset.
 *
 * @param[in]   PERI: specifies peripheral. It can be any combination of the following values:
 *              @arg RCU_RST_AFIO (AF reset)
 *              @arg RCU_RST_GPIOA (GPIOA reset)
 *              @arg RCU_RST_GPIOB (GPIOB reset)
 *              @arg RCU_RST_GPIOC (GPIOC reset)
 *              @arg RCU_RST_GPIOD (GPIOD reset)
 *              @arg RCU_RST_GPIOE (GPIOE reset)
 *              @arg RCU_RST_ADC1 (ADC1 reset)
 *              @arg RCU_RST_ADC2 (ADC2 reset)
 *              @arg RCU_RST_TIM1 (TIM1 reset)
 *              @arg RCU_RST_SPI1 (SPI1 reset)
 *              @arg RCU_RST_USART1 (USART1 reset)
 *
 * @param[in]   ENABLE or DISABLE;
 *
 * @return      None.
 */
#define __RCU_APB2_PERI_RST_ENABLE(PERI)         (RCU->APB2RST |= (PERI))

/**@brief       Release the specifies apb2 peripheral reset.
 *
 * @param[in]   PERI: specifies peripheral. It can be any combination of the following values:
 *              @arg RCU_RST_AFIO (AF reset)
 *              @arg RCU_RST_GPIOA (GPIOA reset)
 *              @arg RCU_RST_GPIOB (GPIOB reset)
 *              @arg RCU_RST_GPIOC (GPIOC reset)
 *              @arg RCU_RST_GPIOD (GPIOD reset)
 *              @arg RCU_RST_GPIOE (GPIOE reset)
 *              @arg RCU_RST_ADC1 (ADC1 reset)
 *              @arg RCU_RST_ADC2 (ADC2 reset)
 *              @arg RCU_RST_TIM1 (TIM1 reset)
 *              @arg RCU_RST_SPI1 (SPI1 reset)
 *              @arg RCU_RST_USART1 (USART1 reset)
 *
 * @param[in]   ENABLE or DISABLE;
 *
 * @return      None.
 */
#define __RCU_APB2_PERI_RST_DISABLE(PERI)     (RCU->APB2RST &= ~(PERI))



/**@brief       Enables the specifies apb1 peripheral reset.
 *
 * @param[in]   PERI: specifies peripheral. It can be any combination of the following values:
 *              @arg RCU_RST_TIM2 (TIM2 reset)
 *              @arg RCU_RST_TIM3 (TIM3 reset)
 *              @arg RCU_RST_TIM4 (TIM4 reset)
 *              @arg RCU_RST_WWDT (WWDT reset)
 *              @arg RCU_RST_SPI2 (SPI2 reset)
 *              @arg RCU_RST_USART2 (USART2 reset)
 *              @arg RCU_RST_USART3 (USART3 reset)
 *              @arg RCU_RST_I2C1 (I2C1 reset)
 *              @arg RCU_RST_I2C2 (I2C2 reset)
 *              @arg RCU_RST_USB (USB reset)
 *              @arg RCU_RST_CAN1 (CAN1 reset)
 *              @arg RCU_RST_BAKUP (BKP reset)
 *              @arg RCU_RST_PMU (PMU reset)
 *
 * @return      None.
 */
#define __RCU_APB1_PERI_RST_ENABLE(PERI)         (RCU->APB1RST |= (PERI))

/**@brief       Release the specifies apb1 peripheral reset.
 *
 * @param[in]   PERI: specifies peripheral. It can be any combination of the following values:
 *              @arg RCU_RST_TIM2 (TIM2 reset)
 *              @arg RCU_RST_TIM3 (TIM3 reset)
 *              @arg RCU_RST_TIM4 (TIM4 reset)
 *              @arg RCU_RST_WWDT (WWDT reset)
 *              @arg RCU_RST_SPI2 (SPI2 reset)
 *              @arg RCU_RST_USART2 (USART2 reset)
 *              @arg RCU_RST_USART3 (USART3 reset)
 *              @arg RCU_RST_I2C1 (I2C1 reset)
 *              @arg RCU_RST_I2C2 (I2C2 reset)
 *              @arg RCU_RST_USB (USB reset)
 *              @arg RCU_RST_CAN1 (CAN1 reset)
 *              @arg RCU_RST_BAKUP (BKP reset)
 *              @arg RCU_RST_PMU (PMU reset)
 *
 * @return      None.
 */
#define __RCU_APB1_PERI_RST_DISABLE(PERI)       (RCU->APB1RST &= ~(PERI))  

/**@brief       Backup domain software reset.
 *
 * @param[in]   None.
 *
 * @return      None.
 */
#define __RCU_BAKUP_DOMAIN_RST_ENABLE()         (RCU->VBDC |= RCU_RST_BKPD)

/**@brief       Release backup domain software reset.
 *
 * @param[in]   None.
 *
 * @return      None.
 */
#define __RCU_BAKUP_DOMAIN_RST_DISABLE()        (RCU->VBDC &= ~RCU_RST_BKPD)


/**@brief       Clears the reset flag.
 *
 * @param[in]   None.
 *
 * @return      None.
 */
#define __RCU_RESET_FLAG_CLEAR()            (RCU->STS |= RCU_STS_CRSTF)

/**@brief       Enable the RTC clock.
 *
 * @param[in]   None.
 *
 * @return      None.
 */
#define __RCU_RTC_CLK_ENABLE()              (RCU->VBDC |= RCU_VBDC_RTCCLKEN)

/**@brief       Disable the RTC clock.
 *
 * @param[in]   None.
 *
 * @return      None.
 */
#define __RCU_RTC_CLK_DISABLE()              (RCU->VBDC &= ~RCU_VBDC_RTCCLKEN)


/**@brief      Enables the AHB peripheral clock.
 *              After reset, the peripheral clock (used for registers read/write access) is disabled and the application 
 *              software has to enable this clock before using it.    
 * @param[in]   PERI: specifies the AHB peripheral to gates its clock. It can be any combination of the following values:
 *              @arg RCU_AHB_PERI_CRC (CRC clock)
 *              @arg RCU_AHB_PERI_FMC (has effect only when the Flash memory is in power down mode)  
 *              @arg RCU_AHB_PERI_SRAM (SRAM clock)
 *              @arg RCU_AHB_PERI_DMA (DMA clock)
 *
 * @return      None.
 */
#define __RCU_AHB_CLK_ENABLE(PERI)          (RCU->AHBEN |= (PERI))

/**@brief       Disable the AHB peripheral clock.
 *              After reset, the peripheral clock (used for registers read/write access) is disabled and the application 
 *              software has to enable this clock before using it.    
 * @param[in]   PERI: specifies the AHB peripheral to gates its clock. It can be any combination of the following values:
 *              @arg RCU_AHB_PERI_CRC (CRC clock)
 *              @arg RCU_AHB_PERI_FMC (has effect only when the Flash memory is in power down mode)  
 *              @arg RCU_AHB_PERI_SRAM (SRAM clock)
 *              @arg RCU_AHB_PERI_DMA (DMA clock)
 *
 * @return      None.
 */
#define __RCU_AHB_CLK_DISABLE(PERI)          (RCU->AHBEN &= ~(PERI))

/**@brief       Enables the APB1 peripheral clock.
 *              After reset, the peripheral clock (used for registers read/write access) is disabled and the application 
 *              software has to enable this clock before using it.
 *
 * @param[in]   PERI: specifies the APB1 peripheral to gates its clock. It can be any combination of the following values:
 *              @arg RCU_APB1_PERI_TIM2 (TIM2 clock)
 *              @arg RCU_APB1_PERI_TIM3 (TIM3 clock)
 *              @arg RCU_APB1_PERI_TIM4 (TIM4 clock)
 *              @arg RCU_APB1_PERI_WWDT (WWDT clock)
 *              @arg RCU_APB1_PERI_SPI2 (SPI2 clock)
 *              @arg RCU_APB1_PERI_USART2 (USART2 clock)
 *              @arg RCU_APB1_PERI_USART3 (USART3 clock)
 *              @arg RCU_APB1_PERI_I2C1 (I2C1 clock)
 *              @arg RCU_APB1_PERI_I2C2 (I2C2 clock)
 *              @arg RCU_APB1_PERI_USB (USB clock)
 *              @arg RCU_APB1_PERI_CAN1 (CAN1 clock)
 *              @arg RCU_APB1_PERI_BAKUP (BAKUP clock)
 *              @arg RCU_APB1_PERI_PMU (PMU clock)
 *
 * @return      None.
 */
#define __RCU_APB1_CLK_ENABLE(PERI)     (RCU->APB1EN |= (PERI))

/**@brief       Disables the APB1 peripheral clock.
 *              After reset, the peripheral clock (used for registers read/write access) is disabled and the application 
 *              software has to enable this clock before using it.
 *
 * @param[in]   PERI: specifies the APB1 peripheral to gates its clock. It can be any combination of the following values:
 *              @arg RCU_APB1_PERI_TIM2 (TIM2 clock)
 *              @arg RCU_APB1_PERI_TIM3 (TIM3 clock)
 *              @arg RCU_APB1_PERI_TIM4 (TIM4 clock)
 *              @arg RCU_APB1_PERI_WWDT (WWDT clock)
 *              @arg RCU_APB1_PERI_SPI2 (SPI2 clock)
 *              @arg RCU_APB1_PERI_USART2 (USART2 clock)
 *              @arg RCU_APB1_PERI_USART3 (USART3 clock)
 *              @arg RCU_APB1_PERI_I2C1 (I2C1 clock)
 *              @arg RCU_APB1_PERI_I2C2 (I2C2 clock)
 *              @arg RCU_APB1_PERI_USB (USB clock)
 *              @arg RCU_APB1_PERI_CAN1 (CAN1 clock)
 *              @arg RCU_APB1_PERI_BAKUP (BAKUP clock)
 *              @arg RCU_APB1_PERI_PMU (PMU clock)
 *
 * @return      None.
 */
#define __RCU_APB1_CLK_DISABLE(PERI)     (RCU->APB1EN &= ~(PERI))

/**@brief       Enables the APB2 peripheral clock.
 *              After reset, the peripheral clock (used for registers read/write access) is disabled and the application
 *              software has to enable this clock before using it.
 *
 * @param[in]   PERI: specifies the APB2 peripheral to gates its clock. It can be any combination of the following values:
 *              @arg RCU_APB2_PERI_AFIO (Multi-function IO clock)
 *              @arg RCU_APB2_PERI_GPIOA (GPIOA clock)
 *              @arg RCU_APB2_PERI_GPIOB (GPIOB clock)
 *              @arg RCU_APB2_PERI_GPIOC (GPIOC clock)
 *              @arg RCU_APB2_PERI_GPIOD (GPIOD clock)
 *              @arg RCU_APB2_PERI_GPIOE (GPIOE clock)
 *              @arg RCU_APB2_PERI_ADC1 (ADC1 clock)
 *              @arg RCU_APB2_PERI_ADC2 (ADC2 clock)  
 *              @arg RCU_APB2_PERI_TIM1 (TIM1 clock)
 *              @arg RCU_APB2_PERI_SPI1 (SPI1 clock)
 *              @arg RCU_APB2_PERI_USART1 (USART1 clock)
 *
 * @return      None.
 */
#define __RCU_APB2_CLK_ENABLE(PERI)     (RCU->APB2EN |= (PERI))

/**@brief       Disables the APB2 peripheral clock.
 *              After reset, the peripheral clock (used for registers read/write access) is disabled and the application
 *              software has to enable this clock before using it.
 *
 * @param[in]   PERI: specifies the APB2 peripheral to gates its clock. It can be any combination of the following values:
 *              @arg RCU_APB2_PERI_AFIO (Multi-function IO clock)
 *              @arg RCU_APB2_PERI_GPIOA (GPIOA clock)
 *              @arg RCU_APB2_PERI_GPIOB (GPIOB clock)
 *              @arg RCU_APB2_PERI_GPIOC (GPIOC clock)
 *              @arg RCU_APB2_PERI_GPIOD (GPIOD clock)
 *              @arg RCU_APB2_PERI_GPIOE (GPIOE clock)
 *              @arg RCU_APB2_PERI_ADC1 (ADC1 clock)
 *              @arg RCU_APB2_PERI_ADC2 (ADC2 clock)  
 *              @arg RCU_APB2_PERI_TIM1 (TIM1 clock)
 *              @arg RCU_APB2_PERI_SPI1 (SPI1 clock)
 *              @arg RCU_APB2_PERI_USART1 (USART1 clock)
 *
 * @return      None.
 */
#define __RCU_APB2_CLK_DISABLE(PERI)    (RCU->APB2EN &= ~(PERI))

/**@brief       Resets the RCU clock configuration to the default reset state.
 *              The default reset state including HRC ON and used as system clock source HRC, HXT and PLL OFF AHB, APB 
 *              prescaler is set to 1, CLKOUT is off.  All interrupts is disabled, but it doesn't modify the
 *              configuration of the peripheral clocks LRC, LXT and RTC clocks.
 *
 * @param       None.
 *
 * @return      None.
 */
void rcu_def_init(void);

/**@brief       Waits for HXT stabilization. It waits on HSERDY flag to be set and return SUCCESS. The HXT is stopped by  
 *              hardware when entering STOP and STANDBY modes.
 *
 * @param       None.
 *
 * @return      SUCCESS(oscillator is stable) or ERROR(oscillator not yet stable).
 */
error_status_t rcu_hxt_stabilization_wait(void);

/** @brief      Set external oscillator(HXT) status.
 *
 * @param[in]   HXT status,it can be one of the following values: @ref RCU_hxt_state.
 *
 * @return      None.
 */
void rcu_hxt_status_set(uint32_t rcu_hxt);

/**@brief       Adjusts the internal high speed RC (HRC) calibration value.
 *
 * @param[in]   value: specifies the HRC calibration trimming value (between 0 and 0x1F).
 *
 * @return      None.
 */
void rcu_hrc_calibration_adjust(uint8_t value);

/**@brief       Configures the PLL clock source and multiplition factor.It must be used when the PLL is disabled.
 *
 * @param[in]   source: specifies the PLL selected clock source,it can be one of the following values: @ref RCU_pll_src.
 *
 * @param[in]   multiple: specifies the PLL multiplication factor,it can be one of the following values: @ref RCU_pll_multiple.
 *
 * @return      None.
 */
void rcu_pll_config(uint32_t source, uint32_t multiple);

/**@brief       Selects the clock source to output on CLKOUT pin.
 *
 * @param[in]   source: specifies the clock source to output, it can be one of the following values: @ref RCU_clk_out.
 *
 * @return      None.
 */
void rcu_clkout_config(uint32_t source);

/**@brief       Configures the system clock (SYSCLK).
 *
 * @param[in]   source: specifies the clock source used as system clock. It can be one of the following values: @ref RCU_sysclk_source
 *
 * @return      None.
 */
void rcu_sysclk_config(uint32_t source);

/**@brief       Get the system clock (SYSCLK) source.
 *
 * @param[in]   None. 
 *
 * @return      sysclk source,It can be one of the following values: @ref RCU_sysclk_source
 */
uint32_t rcu_sysclk_src_get(void);

/**@brief       Configures the APB1 clock.
 *
 * @param[in]   source: HCLK divided used as APB1 clock. It can be one of the following values: @ref RCU_pclk_source
 *
 * @return      None.
 */
void rcu_pclk1_config(uint32_t source);

/**@brief       Configures the APB2 clock.
 *
 * @param[in]   source: HCLK divided used as APB2 clock. It can be one of the following values: @ref RCU_pclk_source
 *
 * @return      None.
 */
void rcu_pclk2_config(uint32_t source);

/**@brief       Configures the AHB clock.
 *
 * @param[in]   source: SYSCLK divided used as AHB clock. It can be one of the following values: @ref RCU_pclk_source
 *
 * @return      None.
 */
void rcu_hclk_config(uint32_t source);

/**@brief       Configures the ADC clock.
 *
 * @param[in]   source: APB2 clock divided used as adc clock. It can be one of the following values: @ref RCU_adcclk_source
 *
 * @return      None.
 */
void rcu_adcclk_config(uint32_t source);

/**@brief       Configures the USB clock.
 *
 * @param[in]   source: PLL clock divided used as USB clock. It can be one of the following values: @ref RCU_usbclk_source
 *
 * @return      None.
 */
void rcu_usbclk_config(uint32_t source);

/**@brief       Configures the RTC clock.
 *
 * @param[in]   source: It can be one of the following values: @ref RCU_rtcclk_source
 *
 * @return      None.
 */
void rcu_rtcclk_config(uint32_t source);

/**@brief       Get RCU clock stabilization and reset flag.
 *
 * @param[in]   flag: specifies the flag to get. It can be one of the following values: @ref RCU_clk_ready_reset_flag
 *
 * @param[out]  status: the flag is SET or RESET
 *
 * @return      SUCCESS OR ERROR.
 */
bit_status_t rcu_clkready_reset_flag_get(uint8_t flag);

/**@brief       Returns the frequencies of the System, AHB and APB busses clocks.
 *
 * @param[in]   ptr_clock: pointer to a rcu_clock_t structure which will hold the clocks frequencies.
 *
 * @return      None.
 */
void rcu_clk_freq_get(rcu_clock_t *ptr_clock);

/*@}*/ /* end of group RCU */


#ifdef __cplusplus
}
#endif

#endif

/*** (C) COPYRIGHT 2021 Software Development . ***/

