/***************************************************************************//**
 * @file        cs32f10x_can.c
 * @version     V2.0.1
 * @author      Software Development
 * @brief       This file contains the definition of CAN function parameters and function interface declaration.
 *              
 * @copyright   Copyright (C) Software Development. All rights reserved.
****************************************************************************/


/*****************************************************************************
 * @includes 
*****************************************************************************/
#include "cs32f10x_can.h"
#include "cs32f10x_rcu.h"



/*****************************************************************************
 * @definitions 
*****************************************************************************/

#define INIT_COMP_TIME_OUT          ((uint32_t)0xFFFF)      /* Time out for initialization complete */

#define CAN_TX_SUCCESS              ((uint8_t)0x00)         /* CAN tx success */
#define CAN_TX_FAILED               ((uint8_t)0x01)         /* CAN tx failed */
#define CAN_TX_PENDING              ((uint8_t)0x02)         /* CAN tx pengding */
#define CAN_TX_MAILBOXFULL          ((uint8_t)0x04)         /* CAN tx mailbox all full */

#define CAN_SLEEP_SUCCESS           ((uint8_t)0x00)         /* CAN sleep success */
#define CAN_SLEEP_FAILED            ((uint8_t)0x01)         /* CAN sleep failed */

#define CAN_TIMEOUT                 ((uint32_t)0x0000FFFF)  /* CAN sleep timeout value */
#define CAN_WKUP_SUCCESS            ((uint8_t)0x00)         /* CAN wakeup success */
#define CAN_WKUP_FAILED             ((uint8_t)0x01)         /* CAN wakeup success */

#define CAN_MODE_SWITCH_SUCCESS     ((uint8_t)0x00)
#define CAN_MODE_SWITCH_FAILED      ((uint8_t)0x01)


/*****************************************************************************
 * @functions
*****************************************************************************/

/**@brief       Initializes the specified CAN peripheral according to the specified parameters in the
 *              ptr_can_config.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It can be CAN1. 
 * 
 * @param[in]   ptr_can_config: Pointer to a can_config_t structure containing the configuration for the specified
 *              CAN peripheral. 
 * 
 * @return      Status of function called, it could be ERROR or SUCCESS.
 */
error_status_t can_init(can_reg_t *ptr_can, can_config_t *ptr_can_config)
{
    uint32_t time_out = 0;
    error_status_t ret = ERROR;

    ASSERT(CAN_LIST_CHECK(ptr_can));
    ASSERT(CAN_MODE_CHECK(ptr_can_config->can_mode));
    ASSERT(CAN_PDIV_CHECK(ptr_can_config->baudrate_pdiv));
    ASSERT(CAN_TIME_CHANGE_UNIT_CHECK(ptr_can_config->time_change));
    ASSERT(CAN_TS1_UNIT_CHECK(ptr_can_config->time_seg1));
    ASSERT(CAN_TS2_UNIT_CHECK(ptr_can_config->time_seg2));
    ASSERT(FUNCTIONAL_STATE_CHECK(ptr_can_config->time_trigger));
    ASSERT(FUNCTIONAL_STATE_CHECK(ptr_can_config->auto_busoff));
    ASSERT(FUNCTIONAL_STATE_CHECK(ptr_can_config->auto_active));
    ASSERT(FUNCTIONAL_STATE_CHECK(ptr_can_config->no_retransmit));
    ASSERT(FUNCTIONAL_STATE_CHECK(ptr_can_config->rx_overwrite));
    ASSERT(FUNCTIONAL_STATE_CHECK(ptr_can_config->tx_priority));

    /* Do not enter sleep mode */
    ptr_can->MCTR &= ~CAN_MCTR_SLEEP;

    /* Initialization request mode */
    ptr_can->MCTR |= CAN_MCTR_INRQ;

    /* Wait until Initialization request complete */
    while(((ptr_can->MSTS & CAN_MSTS_INITF) != CAN_MSTS_INITF) && (time_out < INIT_COMP_TIME_OUT))
    {
        time_out ++;
    }

    /* Judge initialization */
    if((ptr_can->MSTS & CAN_MSTS_INITF) != CAN_MSTS_INITF) /* initialization fail */
    {
        ret = ERROR;
    }
    else                                            /* initialization success */
    {
        /* Set time trigger communication */
        if(ptr_can_config->time_trigger == ENABLE)
        {
            ptr_can->MCTR |= CAN_MCTR_TTCEN;
        }
        else
        {
            ptr_can->MCTR &= ~CAN_MCTR_TTCEN;
        }

        /* Set automatic bus-off management */
        if(ptr_can_config->auto_busoff == ENABLE)
        {
            ptr_can->MCTR |= CAN_MCTR_AUTOFFM;
        }
        else
        {
            ptr_can->MCTR &= ~CAN_MCTR_AUTOFFM;
        }

        /* Set automatic activation */
        if(ptr_can_config->auto_active == ENABLE)
        {
            ptr_can->MCTR |= CAN_MCTR_AUTOATV;
        }
        else
        {
            ptr_can->MCTR &= ~CAN_MCTR_AUTOATV;
        }

        /* Set No automotic retransmission */
        if(ptr_can_config->no_retransmit == ENABLE)
        {
            ptr_can->MCTR |= CAN_MCTR_NAUTORT;
        }
        else
        {
            ptr_can->MCTR &= ~CAN_MCTR_NAUTORT;
        }

        /* Set Receive FIFO over-write mode */
        if(ptr_can_config->rx_overwrite == ENABLE)
        {
            ptr_can->MCTR |= CAN_MCTR_RFOWRM;
        }
        else
        {
            ptr_can->MCTR &= ~CAN_MCTR_RFOWRM;
        }

        /* Set Transmit FIFO priority */
        if(ptr_can_config->tx_priority == ENABLE)
        {
            ptr_can->MCTR |= CAN_MCTR_TXFP;
        }
        else
        {
            ptr_can->MCTR &= ~CAN_MCTR_TXFP;
        }

        /* Set bit timing register */
        ptr_can->BTM = ptr_can_config->can_mode | ptr_can_config->time_change | \
                    ptr_can_config->time_seg1 | ptr_can_config->time_seg2 | \
                    (ptr_can_config->baudrate_pdiv - 1);

        /* exit inilization mode */
        ptr_can->MCTR &= ~CAN_MCTR_INRQ;

        /* Wait ACK */
        time_out = 0;
        while(((ptr_can->MSTS & CAN_MSTS_INITF) != CAN_MSTS_INITF) && (time_out < INIT_COMP_TIME_OUT))
        {
            time_out ++;
        }

        /* check ACK */
        if((ptr_can->MSTS & CAN_MSTS_INITF) != CAN_MSTS_INITF) /* initialization fail */
        {
            ret = ERROR;
        }
        else                                            /* initialization success */
        {
            ret = SUCCESS;
        }
    }

    return ret;
}

/**@brief       Config each ptr_can_config member with its default value.
 *
 * @param[in]   ptr_can_config: Pointer to a can_config_t structure containing the configuration for the specified
 *              CAN peripheral. 
 * 
 * @return      None.
 */
void can_config_struct_init(can_config_t *ptr_can_config)
{
    ptr_can_config->can_mode        = CAN_MODE_NORMAL;
    ptr_can_config->baudrate_pdiv   = 1;
    ptr_can_config->time_change     = CAN_TIME_CHANGE_UNIT_1;
    ptr_can_config->time_seg1       = CAN_TS1_UNIT_4;
    ptr_can_config->time_seg2       = CAN_TS2_UNIT_4;
    ptr_can_config->time_trigger    = DISABLE;
    ptr_can_config->auto_busoff     = DISABLE;
    ptr_can_config->auto_active     = DISABLE;
    ptr_can_config->no_retransmit   = DISABLE;
    ptr_can_config->rx_overwrite    = DISABLE;
    ptr_can_config->tx_priority     = DISABLE;
}

/**@brief       Set can work mode.Only initialization mode could be set
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It can be CAN1.
 * 
 * @param[in]   mode: can work mode.It can be one of the following values: @ref CAN_work_mode
 * 
 * @return      Status of function called, it could be ERROR or SUCCESS.
 */
error_status_t can_work_mode_config(can_reg_t *ptr_can, uint32_t mode)
{
    uint32_t time_out = 0;
    error_status_t ret = ERROR;
    
    ASSERT(CAN_LIST_CHECK(ptr_can));
    ASSERT(CAN_MODE_CHECK(mode));

    /* Initialization request mode */
    ptr_can->MCTR |= CAN_MCTR_INRQ;

    /* Wait until Initialization request complete */
    while(((ptr_can->MSTS & CAN_MSTS_INITF) != CAN_MSTS_INITF) && (time_out < INIT_COMP_TIME_OUT))
    {
        time_out ++;
    }

    /* Judge initialization */
    if((ptr_can->MSTS & CAN_MSTS_INITF) != CAN_MSTS_INITF) /* initialization fail */
    {
        ret = ERROR;
    }
    else                                            /* initialization success */
    {
        MODIFY_REG(ptr_can->BTM, CAN_BTR_SILM | CAN_BTR_LBKM, mode);

        /* exit inilization mode */
        ptr_can->MCTR &= ~CAN_MCTR_INRQ;

        /* Wait ACK */
        time_out = 0;
        while(((ptr_can->MSTS & CAN_MSTS_INITF) != CAN_MSTS_INITF) && (time_out < INIT_COMP_TIME_OUT))
        {
            time_out ++;
        }

        /* check ACK */
        if((ptr_can->MSTS & CAN_MSTS_INITF) != CAN_MSTS_INITF) /* initialization fail */
        {
            ret = ERROR;
        }
        else                                            /* initialization success */
        {
            ret = SUCCESS;
        }
    }

    return ret;
}

/**@brief       Set filter configuration.
 * 
 * @param[in]   ptr_can_filter: Pointer to a specified CAN filter.
 *
 * @return      Status of function called, it could be ERROR or SUCCESS.
 */
void can_filter_init(can_filter_config_t *ptr_can_filter)
{
    uint32_t filter_position = 0;

    ASSERT(CAN_FILTER_CONNECT_FIFO_CHECK(ptr_can_filter->filter_fifo_connect));
    ASSERT(CAN_FILTER_MODE_CHECK(ptr_can_filter->filter_mode));
    ASSERT(CAN_FILTER_WIDTH_CHECK(ptr_can_filter->filter_width));
    ASSERT(FUNCTIONAL_STATE_CHECK(ptr_can_filter->filter_active));
    ASSERT(CAN_FILTER_GROUP_CHECK(ptr_can_filter->filter_group));

    filter_position = ((uint32_t)1) << ptr_can_filter->filter_group;

    /* Initialization FILTER mode */
    CAN1->FLTM |= CAN_FLTM_FWKM;

    /* Deactive filter first */
    CAN1->FAS &= ~filter_position;

    /* Filter mode set */
    if(ptr_can_filter->filter_mode == CAN_FILTER_MODE_IDENTIFY_MASK)
    {
        CAN1->FMOD &= ~filter_position;
    }
    else            /* ID list */
    {
        CAN1->FMOD |= filter_position;
    }

    /* FIFO connection setting */
    if(ptr_can_filter->filter_fifo_connect == CAN_FILTER_CONNECT_FIFO1)
    {
        CAN1->FLTFA |= filter_position;
    }
    else if(ptr_can_filter->filter_fifo_connect == CAN_FILTER_CONNECT_FIFO0)
    {
        CAN1->FLTFA &= ~filter_position;
    }

    /* Set filter width */
    if(ptr_can_filter->filter_width == CAN_FILTER_WIDTH_16BIT)
    {
        /* width select */
        CAN1->FBW &= ~filter_position;

        CAN1->FILTER_REG[ptr_can_filter->filter_group].FR1 = 
            ((ptr_can_filter->filter_mask & 0xFFFF) << 16) |
            (ptr_can_filter->filter_identifier & 0xFFFF);

        CAN1->FILTER_REG[ptr_can_filter->filter_group].FR2 = 
            ((ptr_can_filter->filter_mask & 0xFFFF0000)) |
            ((ptr_can_filter->filter_identifier & 0xFFFF0000) >> 16);
    }
    else if(ptr_can_filter->filter_width == CAN_FILTER_WIDTH_32BIT)
    {
        /* width select */
        CAN1->FBW |= filter_position;

        CAN1->FILTER_REG[ptr_can_filter->filter_group].FR1 = ptr_can_filter->filter_identifier;

        CAN1->FILTER_REG[ptr_can_filter->filter_group].FR2 = ptr_can_filter->filter_mask;
    }

    /* Active filter */
    if(ptr_can_filter->filter_active == ENABLE)
    {
        CAN1->FAS |= filter_position;
    }

    /* exit initialization FILTER mode */
    CAN1->FLTM &= ~CAN_FLTM_FWKM;
}

/**@brief       Enable or disable the CAN time trigger communication mode. If enabled,
 *              time stamp could be send:TIME[15:0] in the last two bytes of the 8-byte
 *              message, TIME[7:0] in the byte 6 and TIME[15:8] in the byte 7.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It can be CAN1.
 * 
 * @param[in]   status: ENABLE or DISABLE.
 * 
 * @return      None.
 */
void can_time_trigger_enable_ctrl(can_reg_t *ptr_can, enable_state_t status)
{
    ASSERT(CAN_LIST_CHECK(ptr_can));
    ASSERT(FUNCTIONAL_STATE_CHECK(status));

    if(status != DISABLE)
    {
        ptr_can->MCTR |= CAN_MCTR_TTCEN;

        ptr_can->TX_MAILBOX[0].TXLT |= CAN_TXLT0_TTS;
        ptr_can->TX_MAILBOX[1].TXLT |= CAN_TXLT1_TTS;
        ptr_can->TX_MAILBOX[2].TXLT |= CAN_TXLT2_TTS;
    }
    else
    {
        ptr_can->MCTR &= ~CAN_MCTR_TTCEN;

        ptr_can->TX_MAILBOX[0].TXLT &= ~CAN_TXLT0_TTS;
        ptr_can->TX_MAILBOX[1].TXLT &= ~CAN_TXLT1_TTS;
        ptr_can->TX_MAILBOX[2].TXLT &= ~CAN_TXLT2_TTS;
    }
}

/**@brief       Initializes the specified CAN transmission configuration according to the specified parameters
 *              in the ptr_msg.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It can be CAN1. 
 * 
 * @param[in]   ptr_msg: Pointer to a can_tx_msg_t structure containing the configuration for the specified
 *              CAN message. 
 * 
 * @return      The number of mailbox or CAN_TX_MAILBOXFULL.
 */
uint8_t can_message_send(can_reg_t *ptr_can, can_tx_msg_t *ptr_msg)
{
    uint32_t mailbox_flag = 0;
    
    ASSERT(CAN_LIST_CHECK(ptr_can));
    ASSERT(CAN_ID_CHECK(ptr_msg->id_select));
    ASSERT(CAN_RMOTE_TX_REQ_CHECK(ptr_msg->remote_txreq));
    ASSERT(CAN_MSG_DATA_LENGTH_CHECK(ptr_msg->data_length));

    /* Find an empty mailbox */
    if((ptr_can->TXSTS & CAN_TXSTS_TME0) == CAN_TXSTS_TME0)
    {
        mailbox_flag = 0;
    }
    else if((ptr_can->TXSTS & CAN_TXSTS_TME1) == CAN_TXSTS_TME1)
    {
        mailbox_flag = 1;
    }
    else if((ptr_can->TXSTS & CAN_TXSTS_TME2) == CAN_TXSTS_TME2)
    {
        mailbox_flag = 2;
    }
    else 
    {
        return CAN_TX_MAILBOXFULL;
    }

    /* mailbox is not empty,config tx mailbox */
    ptr_can->TX_MAILBOX[mailbox_flag].TXMI &= (uint32_t)(0x00000001);

    if(ptr_msg->id_select == CAN_ID_STANDARD)
    {
        ptr_can->TX_MAILBOX[mailbox_flag].TXMI |= (ptr_msg->std_id << 21) | (ptr_msg->remote_txreq);
    }
    else
    {
        ptr_can->TX_MAILBOX[mailbox_flag].TXMI |= (ptr_msg->ext_id << 3) | (ptr_msg->remote_txreq) | (ptr_msg->id_select);
    }

    /* Config data length */
    MODIFY_REG(ptr_can->TX_MAILBOX[mailbox_flag].TXLT, (uint32_t)0x0000000F, ptr_msg->data_length);

    /* Fill the data */
    ptr_can->TX_MAILBOX[mailbox_flag].TXDH = ((uint32_t)ptr_msg->data[7] << 24) | \
                                             ((uint32_t)ptr_msg->data[6] << 16) | \
                                             ((uint32_t)ptr_msg->data[5] << 8)  | \
                                             ((uint32_t)ptr_msg->data[4]);
    
    ptr_can->TX_MAILBOX[mailbox_flag].TXDL = ((uint32_t)ptr_msg->data[3] << 24) | \
                                             ((uint32_t)ptr_msg->data[2] << 16) | \
                                             ((uint32_t)ptr_msg->data[1] << 8)  | \
                                             ((uint32_t)ptr_msg->data[0]);

    ptr_can->TX_MAILBOX[mailbox_flag].TXMI |= (uint32_t)(0x00000001);

    return mailbox_flag;
}

/**@brief       CAN tx status check according to the mailbox number.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It only can be CAN1. 
 * 
 * @param[in]   tx_mailbox: specifies the mailbox
 * 
 * @return      The status of the mailbox send.
 */
uint8_t can_tx_status_get(can_reg_t *ptr_can, uint8_t tx_mailbox)
{
    uint32_t status = 0;
    
    ASSERT(CAN_LIST_CHECK(ptr_can));
    ASSERT(CAN_TX_MAILBOX_NUMBER_CHECK(tx_mailbox));

    if(tx_mailbox == CAN_TX_MAILBOX_NUM0)
    {
        status = ptr_can->TXSTS & (CAN_TXSTS_MRQCP0 | CAN_TXSTS_MTXSU0 | CAN_TXSTS_TME0);
    }
    else if(tx_mailbox == CAN_TX_MAILBOX_NUM1)
    {
        status = ptr_can->TXSTS & (CAN_TXSTS_MRQCP1 | CAN_TXSTS_MTXSU1 | CAN_TXSTS_TME1);
    }
    else if(tx_mailbox == CAN_TX_MAILBOX_NUM2)
    {
        status = ptr_can->TXSTS & (CAN_TXSTS_MRQCP2 | CAN_TXSTS_MTXSU2 | CAN_TXSTS_TME2);
    }
    
    switch(status)
    {
        case 0x0:   
            status = CAN_TX_PENDING;    /* No tx,pending */
            break;
        /* TX failed check */
        case (CAN_TXSTS_MRQCP0 | CAN_TXSTS_TME0):
            status = CAN_TX_FAILED;
            break;
        case (CAN_TXSTS_MRQCP1 | CAN_TXSTS_TME1):
            status = CAN_TX_FAILED;
            break;
        case (CAN_TXSTS_MRQCP2 | CAN_TXSTS_TME2):
            status = CAN_TX_FAILED;
            break;

        /* TX success check */
        case (CAN_TXSTS_MRQCP0 | CAN_TXSTS_MTXSU0 | CAN_TXSTS_TME0):
            status = CAN_TX_SUCCESS;
            break;
        case (CAN_TXSTS_MRQCP1 | CAN_TXSTS_MTXSU1 | CAN_TXSTS_TME1):
            status = CAN_TX_SUCCESS;
            break;
        case (CAN_TXSTS_MRQCP2 | CAN_TXSTS_MTXSU2 | CAN_TXSTS_TME2):
            status = CAN_TX_SUCCESS;
            break;

        default:
            status = CAN_TX_FAILED;
            break;
    }

    return status;
}

/**@brief       CAN tx stop according to the mailbox number.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It can be CAN1. 
 * 
 * @param[in]   tx_mailbox: Specifies the mailbox.
 * 
 * @return      None.
 */
void can_tx_abort_set(can_reg_t *ptr_can, uint8_t tx_mailbox)
{
    ASSERT(CAN_LIST_CHECK(ptr_can));
    ASSERT(CAN_TX_MAILBOX_NUMBER_CHECK(tx_mailbox));

    if(tx_mailbox == CAN_TX_MAILBOX_NUM0)
    {
        ptr_can->TXSTS |= CAN_TXSTS_MATXRQ0;
    }
    else if(tx_mailbox == CAN_TX_MAILBOX_NUM1)
    {
        ptr_can->TXSTS |= CAN_TXSTS_MATXRQ1;
    }
    else
    {
        ptr_can->TXSTS |= CAN_TXSTS_MATXRQ2;
    }
}

/**@brief       CAN receive message according to the rx_fifo.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It only can be CAN1. 
 * 
 * @param[in]   rx_fifo: Specifies the receive fifo number.
 *
 * @param[out]  ptr_msg: Pointer to a can_rx_msg_t structure containing the configuration for the specified
 *              CAN receive message.
 * 
 * @return      None.
 */
void can_message_receive(can_reg_t *ptr_can, uint8_t rx_fifo, can_rx_msg_t *ptr_msg)
{
    ASSERT(CAN_LIST_CHECK(ptr_can));
    ASSERT(CAN_RECEIVE_FIFO_CHECK(rx_fifo));

    /* ID */
    ptr_msg->id_select = ptr_can->RX_FFMAILBOX[rx_fifo].RFFID & 0x04;

    if(ptr_msg->id_select == CAN_ID_STANDARD)
    {
        ptr_msg->std_id = (ptr_can->RX_FFMAILBOX[rx_fifo].RFFID >> 21) & 0x7FF;
    }
    else    /* extend id */
    {
        ptr_msg->ext_id = (ptr_can->RX_FFMAILBOX[rx_fifo].RFFID >> 3) & 0x1FFFFFFF;
    }

    /* Remote tx request */
    ptr_msg->remote_txreq = ptr_can->RX_FFMAILBOX[rx_fifo].RFFID & 0x02;

    /* Information choose control */
    ptr_msg->match_Info = (ptr_can->RX_FFMAILBOX[rx_fifo].RXLT >> 8) & 0xFF;

    /* The length of data */
    ptr_msg->data_length = ptr_can->RX_FFMAILBOX[rx_fifo].RXLT & 0x0F;

    /* Rx data */
    ptr_msg->data[0] = ptr_can->RX_FFMAILBOX[rx_fifo].RXDL & 0xFF;
    ptr_msg->data[1] = (ptr_can->RX_FFMAILBOX[rx_fifo].RXDL >> 8) & 0xFF;
    ptr_msg->data[2] = (ptr_can->RX_FFMAILBOX[rx_fifo].RXDL >> 16) & 0xFF;
    ptr_msg->data[3] = (ptr_can->RX_FFMAILBOX[rx_fifo].RXDL >> 24) & 0xFF;

    ptr_msg->data[4] = ptr_can->RX_FFMAILBOX[rx_fifo].RXDH & 0xFF;
    ptr_msg->data[5] = (ptr_can->RX_FFMAILBOX[rx_fifo].RXDH >> 8) & 0xFF;
    ptr_msg->data[6] = (ptr_can->RX_FFMAILBOX[rx_fifo].RXDH >> 16) & 0xFF;
    ptr_msg->data[7] = (ptr_can->RX_FFMAILBOX[rx_fifo].RXDH >> 24) & 0xFF;

    /* Get Rx data and then release FIFO */
    if(rx_fifo == CAN_RECEIVE_FIFO_0)
    {
        ptr_can->RXFF0 |= CAN_RXFF0_RFOM0;
    }
    else
    {
        ptr_can->RXFF1 |= CAN_RXFF1_RFOM1;
    }
}

/**@brief       CAN FIFO release.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It only can be CAN1. 
 * 
 * @param[in]   rx_fifo: Specifies the receive fifo number.
 *
 * @return      None.
 */
void can_rx_fifo_release(can_reg_t *ptr_can, uint8_t rx_fifo)
{
    ASSERT(CAN_LIST_CHECK(ptr_can));
    ASSERT(CAN_RECEIVE_FIFO_CHECK(rx_fifo));

    if(rx_fifo == CAN_RECEIVE_FIFO_0)
    {
        ptr_can->RXFF0 |= CAN_RXFF0_RFOM0;
    }
    else
    {
        ptr_can->RXFF1 |= CAN_RXFF1_RFOM1;
    }
}

/**@brief       Get CAN FIFO remained message.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It only can be CAN1. 
 * 
 * @param[in]   rx_fifo: Specifies the receive fifo number.
 *
 * @return      The number of FIFO remains.
 */
uint8_t can_fifo_msg_num_get(can_reg_t *ptr_can, uint8_t rx_fifo)
{
    uint8_t msg_number = 0;
    
    ASSERT(CAN_LIST_CHECK(ptr_can));
    ASSERT(CAN_RECEIVE_FIFO_CHECK(rx_fifo));

    if(rx_fifo == CAN_RECEIVE_FIFO_0)
    {
        msg_number = (uint8_t)(ptr_can->RXFF0 & 0x03);
    }
    else
    {
        msg_number = (uint8_t)(ptr_can->RXFF1 & 0x03);
    }

    return msg_number;
}

/**@brief       Set CAN sleep mode.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It only can be CAN1. 
 * 
 * @return      The status of sleep mode.
 */
uint8_t can_sleep_set(can_reg_t *ptr_can)
{
    uint8_t status = CAN_SLEEP_FAILED;
    uint32_t time_out = 0;

    ASSERT(CAN_LIST_CHECK(ptr_can));

    /* Enable sleep request */
    ptr_can->MCTR = (ptr_can->MCTR & (~CAN_MCTR_INRQ)) | CAN_MCTR_SLEEP;

    /* Confirm sleep status with timeout */
    while(((ptr_can->MSTS & CAN_MSTS_SLEEPF) != CAN_MSTS_SLEEPF) && (time_out < CAN_TIMEOUT))
    {
        time_out ++;
    }
    
    /* Confirm sleep status */
    if((ptr_can->MSTS & CAN_MSTS_SLEEPF) == CAN_MSTS_SLEEPF)
    {
        status = CAN_SLEEP_SUCCESS;
    }

    return status;
}

/**@brief       Set CAN wakeup mode.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It only can be CAN1. 
 * 
 * @return      The status of wakeup mode.
 */
uint8_t can_wakeup_set(can_reg_t *ptr_can)
{
    uint8_t status = CAN_WKUP_FAILED;
    uint32_t time_out = 0;

    ASSERT(CAN_LIST_CHECK(ptr_can));

    /* Disable sleep request */
    ptr_can->MCTR &= ~CAN_MCTR_SLEEP;

    /* Confirm sleep status with timeout */
    while(((ptr_can->MSTS & CAN_MSTS_SLEEPF) == CAN_MSTS_SLEEPF) && (time_out < CAN_TIMEOUT))
    {
        time_out ++;
    }

    /* Confirm again */
    if((ptr_can->MSTS & CAN_MSTS_SLEEPF) != CAN_MSTS_SLEEPF)
    {
        status = CAN_WKUP_SUCCESS;
    }

    return status;
}

/**@brief       Set CAN operation mode:sleep,initialization and normal.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It only can be CAN1. 
 * 
 * @param[in]   mode: CAN operation mode. It can be one of the following values: @ref CAN_operation_mode
 *
 * @return      The status of mode switch.
 */
uint8_t can_op_mode_switch_ctrl(can_reg_t *ptr_can, uint8_t mode)
{
    uint8_t status = CAN_MODE_SWITCH_FAILED;
    uint32_t time_out = 0;
    
    ASSERT(CAN_LIST_CHECK(ptr_can));
    ASSERT(CAN_OP_MODE_CHECK(mode));

    if(mode == CAN_OP_MODE_SLEEP)
    {
        /* Enable sleep request */
        ptr_can->MCTR = (ptr_can->MCTR & (~CAN_MCTR_INRQ)) | CAN_MCTR_SLEEP;

        /* Confirm sleep status with timeout */
        while(((ptr_can->MSTS & CAN_MSTS_SLEEPF) != CAN_MSTS_SLEEPF) && (time_out < CAN_TIMEOUT))
        {
            time_out ++;
        }
        
        /* Confirm sleep status */
        if((ptr_can->MSTS & CAN_MSTS_SLEEPF) == CAN_MSTS_SLEEPF)
        {
            status = CAN_MODE_SWITCH_SUCCESS;
        }
    }
    else if(mode == CAN_OP_MODE_NORMAL)
    {
        /* Exit sleep and initialization request */
        ptr_can->MCTR &= ~(CAN_MCTR_INRQ | CAN_MCTR_SLEEP);

        /* Confirm status with timeout */
        while(((ptr_can->MSTS & (CAN_MSTS_SLEEPF | CAN_MSTS_INITF)) != 0) && (time_out < CAN_TIMEOUT))
        {
            time_out ++;
        }

        if((ptr_can->MSTS & (CAN_MSTS_SLEEPF | CAN_MSTS_INITF)) == 0)
        {
            status = CAN_MODE_SWITCH_SUCCESS;
        }
    }
    else if(mode == CAN_OP_MODE_INITIAL)
    {
        /* initialization request */
        ptr_can->MCTR = (ptr_can->MCTR & (~CAN_MCTR_SLEEP)) | CAN_MCTR_INRQ;

        /* Confirm status with timeout */
        while(((ptr_can->MSTS & (CAN_MSTS_SLEEPF | CAN_MSTS_INITF)) != CAN_MSTS_INITF) && (time_out < CAN_TIMEOUT))
        {
            time_out ++;
        }

        if((ptr_can->MSTS & (CAN_MSTS_SLEEPF | CAN_MSTS_INITF)) == CAN_MSTS_INITF)
        {
            status = CAN_MODE_SWITCH_SUCCESS;
        }
    }
    else
    {
        status = CAN_MODE_SWITCH_FAILED;
    }

    return status;
}

/**@brief       Get CAN flag according to registers.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It only can be CAN1. 
 * 
 * @param[in]   flag: Specifies the flag.
 *              CAN_FLAG_MRQCP0 (Mailbox0 Request completed flag)
 *              CAN_FLAG_MRQCP1 (Mailbox1 Request completed flag)
 *              CAN_FLAG_MRQCP2 (Mailbox2 Request completed flag)
 *              CAN_FLAG_FIFOMP1 (FIFO 1 message pengding)
 *              CAN_FLAG_FIFOFULL1 (FIFO 1 is full)
 *              CAN_FLAG_FIFOOVR1 (FIFO 1 overrun)
 *              CAN_FLAG_FIFOMP0 (FIFO 0 message pengding)
 *              CAN_FLAG_FIFOFULL0 (FIFO 0 is full)
 *              CAN_FLAG_FIFOOVR0 (FIFO 0 overrun)
 *              CAN_FLAG_SLEEPF ( Sleep flag)
 *              CAN_FLAG_WKUI (Wakeup flag)
 *              CAN_FLAG_ERRWF (Error Warning Flag)
 *              CAN_FLAG_ERRPF (Error passive flag)
 *              CAN_FLAG_BOFF (Bus-off flag)
 *              CAN_FLAG_LEC (Last error code Flag)
 *
 * @return      The status of flag.(SET or REST)
 */
bit_status_t can_flag_status_get(can_reg_t *ptr_can, uint32_t flag)
{
    bit_status_t status = RESET;

    ASSERT(CAN_LIST_CHECK(ptr_can));
    ASSERT(CAN_GET_FLAG_CHECK(flag));

    if((flag & CAN_FLAG_IN_TXSTS_REG) != 0)
    {
        if((ptr_can->TXSTS & (flag & 0x000FFFFF)) != 0)
        {
            status = SET;
        }
        else
        {
            status = RESET;
        }
    }
    else if((flag & CAN_FLAG_IN_RXFF1_REG) != 0)
    {
        if((ptr_can->RXFF1 & (flag & 0x000FFFFF)) != 0)
        {
            status = SET;
        }
        else
        {
            status = RESET;
        }
    }
    else if((flag & CAN_FLAG_IN_RXFF0_REG) != 0)
    {
        if((ptr_can->RXFF0 & (flag & 0x000FFFFF)) != 0)
        {
            status = SET;
        }
        else
        {
            status = RESET;
        }
    }
    else if((flag & CAN_FLAG_IN_MSTS_REG) != 0)
    {
        if((ptr_can->MSTS & (flag & 0x000FFFFF)) != 0)
        {
            status = SET;
        }
        else
        {
            status = RESET;
        }
    }
    else if((flag & CAN_FLAG_IN_ERRSTS_REG) != 0)
    {
        if((ptr_can->ERRSTS & (flag & 0x000FFFFF)) != 0)
        {
            status = SET;
        }
        else
        {
            status = RESET;
        }
    }

    return status;
}

/**@brief       Clear CAN flag.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It only can be CAN1. 
 * 
 * @param[in]   flag: Specifies the flag.
 *              CAN_FLAG_MRQCP0 (Mailbox0 Request completed flag)
 *              CAN_FLAG_MRQCP1 (Mailbox1 Request completed flag)
 *              CAN_FLAG_MRQCP2 (Mailbox2 Request completed flag)
 *              CAN_FLAG_FIFOFULL1 (FIFO 1 is full)
 *              CAN_FLAG_FIFOOVR1 (FIFO 1 overrun)
 *              CAN_FLAG_FIFOFULL0 (FIFO 0 is full)
 *              CAN_FLAG_FIFOOVR0 (FIFO 0 overrun)
 *              CAN_FLAG_SLEEPF ( Sleep flag)
 *              CAN_FLAG_WKUI (Wakeup flag)
 *              CAN_FLAG_LEC (Last error code Flag)
 *
 * @return      The status of flag.(SET or REST)
 */
void can_flag_clear(can_reg_t *ptr_can, uint32_t flag)
{
    ASSERT(CAN_LIST_CHECK(ptr_can));
    ASSERT(CAN_CLEAR_FLAG_CHECK(flag));

    if((flag & CAN_FLAG_IN_TXSTS_REG) != 0)
    {
        ptr_can->TXSTS = flag & 0x000FFFFF;
    }
    else if((flag & CAN_FLAG_IN_RXFF1_REG) != 0)
    {
        ptr_can->RXFF1 = flag & 0x000FFFFF;
    }
    else if((flag & CAN_FLAG_IN_RXFF0_REG) != 0)
    {
        ptr_can->RXFF0 = flag & 0x000FFFFF;
    }
    else if((flag & CAN_FLAG_IN_MSTS_REG) != 0)
    {
        ptr_can->MSTS = flag & 0x000FFFFF;
    }
    else if((flag == CAN_FLAG_LEC) != 0)
    {
        ptr_can->ERRSTS = (uint32_t)0x0;
    }
}

/**@brief       Get CAN interrupt flag.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It only can be CAN1. 
 * 
 * @param[in]   int_flag: Specifies the flag.
 *              CAN_INTR_TX_EMPTY
 *              CAN_INTR_FF0_MSG_PENDING
 *              CAN_INTR_FF0_FULL
 *              CAN_INTR_FF0_OVRRUN
 *              CAN_INTR_FF1_MSG_PENDING
 *              CAN_INTR_FF1_FULL
 *              CAN_INTR_FF1_OVERRUN
 *              CAN_INTR_ERR_WARNING
 *              CAN_INTR_ERR_PASSIVE
 *              CAN_INTR_BUS_OFF
 *              CAN_INTR_LAST_ERR_NUMBER
 *              CAN_INTR_ERROR
 *              CAN_INTR_WAKEUP
 *              CAN_INTR_SLEEP
 *
 * @return      The status of flag.(SET or RESET)
 */
bit_status_t can_int_status_get(can_reg_t *ptr_can, uint32_t int_flag)
{
    bit_status_t status = RESET;
    
    ASSERT(CAN_LIST_CHECK(ptr_can));
    ASSERT(CAN_INTR_FLAG_CHECK(int_flag));

    if((ptr_can->INTR & int_flag) != 0)
    {
        switch(int_flag)
        {
            case CAN_INTR_TX_EMPTY:
            {
                status = (ptr_can->TXSTS & (CAN_TXSTS_MRQCP0|CAN_TXSTS_MRQCP1|CAN_TXSTS_MRQCP2)) ? SET:RESET;
            }
            break;

            case CAN_INTR_FF0_MSG_PENDING:
            {
                status = (ptr_can->RXFF0 & CAN_RXFF0_FIFOMP0) ? SET:RESET;
            }
            break;

            case CAN_INTR_FF0_FULL:
            {
                status = (ptr_can->RXFF0 & CAN_RXFF0_FIFOF0) ? SET:RESET;
            }
            break;

            case CAN_INTR_FF0_OVRRUN:
            {
                status = (ptr_can->RXFF0 & CAN_RXFF0_FIFOVR0) ? SET:RESET;
            }
            break;

            case CAN_INTR_FF1_MSG_PENDING:
            {
                status = (ptr_can->RXFF1 & CAN_RXFF1_FIFOMP1) ? SET:RESET;
            }
            break;

            case CAN_INTR_FF1_FULL:
            {
                status = (ptr_can->RXFF1 & CAN_RXFF1_FIFOF1) ? SET:RESET;
            }
            break;

            case CAN_INTR_FF1_OVERRUN:
            {
                status = (ptr_can->RXFF1 & CAN_RXFF1_FIFOVR1) ? SET:RESET;
            }
            break;

            case CAN_INTR_ERR_WARNING:
            {
                status = (ptr_can->ERRSTS & CAN_ERRSTS_ERRWF) ? SET:RESET;
            }
            break;

            case CAN_INTR_ERR_PASSIVE:
            {
                status = (ptr_can->ERRSTS & CAN_ERRSTS_ERRPF) ? SET:RESET;
            }
            break;

            case CAN_INTR_BUS_OFF:
            {
                status = (ptr_can->ERRSTS & CAN_ERRSTS_BOFF) ? SET:RESET;
            }
            break;

            case CAN_INTR_LAST_ERR_NUMBER:
            {
                status = (ptr_can->ERRSTS & CAN_ERRSTS_LEC) ? SET:RESET;
            }
            break;

            case CAN_INTR_ERROR:
            {
                status = (ptr_can->MSTS & CAN_MSTS_ERI) ? SET:RESET;
            }
            break;

            case CAN_INTR_WAKEUP:
            {
                status = (ptr_can->MSTS & CAN_MSTS_WKUI) ? SET:RESET;
            }
            break;

            case CAN_INTR_SLEEP:
            {
                status = (ptr_can->MSTS & CAN_MSTS_SEI) ? SET:RESET;
            }
            break;

            default:
                status = RESET;
        }
    }
    else
    {
        status = RESET;
    }

    return status;
}

/**@brief       Clear CAN interrupt flag.
 * 
 * @param[in]   ptr_can: Pointer to a specified CAN peripheral. It only can be CAN1. 
 * 
 * @param[in]   int_flag: Specifies the flag.
 *              CAN_INTR_TX_EMPTY
 *              CAN_INTR_FF0_FULL
 *              CAN_INTR_FF0_OVRRUN
 *              CAN_INTR_FF1_FULL
 *              CAN_INTR_FF1_OVERRUN
 *              CAN_INTR_ERR_WARNING
 *              CAN_INTR_ERR_PASSIVE
 *              CAN_INTR_BUS_OFF
 *              CAN_INTR_LAST_ERR_NUMBER
 *              CAN_INTR_ERROR
 *              CAN_INTR_WAKEUP
 *              CAN_INTR_SLEEP
 *
 * @return      The status of flag.(SET or RESET)
 */
void can_int_flag_clear(can_reg_t *ptr_can, uint32_t clear_flag)
{
    ASSERT(CAN_LIST_CHECK(ptr_can));
    ASSERT(CAN_INTR_CLEAR_CHECK(clear_flag));

    switch(clear_flag)
    {
        case CAN_INTR_TX_EMPTY:
        {
            ptr_can->TXSTS = CAN_TXSTS_MRQCP0|CAN_TXSTS_MRQCP1|CAN_TXSTS_MRQCP2;
        }
        break;

        case CAN_INTR_FF0_FULL:
        {
            ptr_can->RXFF0 = CAN_RXFF0_FIFOF0;
        }
        break;

        case CAN_INTR_FF0_OVRRUN:
        {
            ptr_can->RXFF0 = CAN_RXFF0_FIFOVR0;
        }
        break;

        case CAN_INTR_FF1_FULL:
        {
            ptr_can->RXFF1 = CAN_RXFF1_FIFOF1;
        }
        break;

        case CAN_INTR_FF1_OVERRUN:
        {
            ptr_can->RXFF1 = CAN_RXFF1_FIFOVR1;
        }
        break;

        case CAN_INTR_ERR_WARNING:
        {
            ptr_can->MSTS = CAN_MSTS_ERI;
        }
        break;

        case CAN_INTR_ERR_PASSIVE:
        {
            ptr_can->MSTS = CAN_MSTS_ERI;
        }
        break;

        case CAN_INTR_BUS_OFF:
        {
            ptr_can->MSTS = CAN_MSTS_ERI;
        }
        break;

        case CAN_INTR_LAST_ERR_NUMBER:
        {
            ptr_can->ERRSTS = 0x0;
            ptr_can->MSTS = CAN_MSTS_ERI;
        }
        break;

        case CAN_INTR_ERROR:
        {
            ptr_can->ERRSTS = 0x0;
            ptr_can->MSTS = CAN_MSTS_ERI;
        }
        break;

        case CAN_INTR_WAKEUP:
        {
            ptr_can->MSTS = CAN_MSTS_WKUI;
        }
        break;

        case CAN_INTR_SLEEP:
        {
            ptr_can->MSTS = CAN_MSTS_SEI;
        }
        break;

        default:
        break;
    }
}

/*** (C) COPYRIGHT 2021 Software Development. ***/


