/**************************************************************************//**
 * @file     cs32f10x_gpio.c
 * @version  V2.0.1
 * @author   Software Development
 * @brief    nothing
 * 
 * @copyright Copyright (C) Software Development. All rights reserved.
 *****************************************************************************/

/*****************************************************************************
 * @includes 
*****************************************************************************/
#include "cs32f10x_gpio.h"
#include "cs32f10x_rcu.h"


/*****************************************************************************
 * @definitions 
*****************************************************************************/
#define GPIO_IN_OUT_MODE_MASK            0x40        /* bit6   - 1:OUT_MODE, 0:IN_MODE */


/*****************************************************************************
 * @functions
*****************************************************************************/

/**@brief       Sets the specified IO as specified mode.
 *
 * @param[in]   ptr_gpio: pointer to GPIOx, where x can be A to E .
 *
 * @param[in]   pin: specifies the port pin to be set (Support any combination of GPIO_PIN_0 to GPIO_PIN_15).
 *
 * @param[in]   mode: Mode selection. This parameter can be one of @ref GPIO_mode_selection.
 *
 * @return      None.
 */
void gpio_mode_config(gpio_reg_t * ptr_gpio, uint16_t pin, uint32_t mode)
{
    uint32_t reg_temp = 0x0,pin_temp = 0x0;
    uint32_t pin_pos = 0x0;
    
    ASSERT(GPIO_ALL_PORT_CHECK(ptr_gpio));
    ASSERT(GPIO_PIN_CHECK(pin));
    ASSERT(GPIO_MODE_CHECK(mode));

    if((pin & 0x00FF) != 0x0)       /* GPIO_PIN_0 ~ GPIO_PIN_7 */
    {
        reg_temp = ptr_gpio->CFGL;

        for(pin_temp = 0x0; pin_temp < 8; pin_temp ++)
        {
            if(pin & (1 << pin_temp))
            {
                pin_pos = pin_temp << 2 ;
                if((mode & GPIO_IN_OUT_MODE_MASK) == 0x0)    /* Input mode */
                {
                    /* Clear bits */
                    reg_temp &= ~((uint32_t)0x0F << pin_pos);
                    /* Set mode */
                    reg_temp |= (mode & ((uint32_t)0x0F)) << pin_pos;

                    if(mode == GPIO_MODE_IN_PD)
                    {
                        ptr_gpio->CLRR = ((uint32_t)0x01) << pin_temp;
                    }
                    else if(mode == GPIO_MODE_IN_PU)
                    {
                        ptr_gpio->SCR = ((uint32_t)0x01) << pin_temp;
                    }
                }
                else                                        /* Out mode */
                {
                    /* Clear bits */
                    reg_temp &= ~((uint32_t)0x0F << pin_pos);
                    /* Set mode */
                    reg_temp |= (mode & ((uint32_t)0x0F)) << pin_pos;
                }
            }
        }
        ptr_gpio->CFGL = reg_temp;
    }
    
    if((pin & 0xFF00) != 0x0)       /* GPIO_PIN_8 ~ GPIO_PIN_15 */
    {
        reg_temp = ptr_gpio->CFGH;

        for(pin_temp = 0x0; pin_temp < 8; pin_temp ++)
        {
            if(pin & (1 << (pin_temp + 0x08)))
            {
                pin_pos = pin_temp << 2;
                if((mode & GPIO_IN_OUT_MODE_MASK) == 0x0)    /* Input mode */
                {
                    /* Clear bits */
                    reg_temp &= ~((uint32_t)0x0F << pin_pos);
                    /* Set mode */
                    reg_temp |= (mode & ((uint32_t)0x0F)) << pin_pos;

                    if(mode == GPIO_MODE_IN_PD)
                    {
                        ptr_gpio->CLRR = (((uint32_t)0x01) << (pin_temp + 0x08)) ;
                    }
                    else if(mode == GPIO_MODE_IN_PU)
                    {
                        ptr_gpio->SCR = (((uint32_t)0x01) << (pin_temp + 0x08));
                    }
                }
                else                                        /* Out mode */
                {
                    /* Clear bits */
                    reg_temp &= ~((uint32_t)0x0F << pin_pos);
                    /* Set mode */
                    reg_temp |= (mode & ((uint32_t)0x0F)) << pin_pos;
                }
            }
        }
        ptr_gpio->CFGH = reg_temp;
    }
    
}

/**@brief       Reset GPIOx registers to default value.
 *
 * @param[in]   ptr_gpio: pointer to GPIOx, where x can be A to E .
 *
 * @return      None.
 */
void gpio_def_init(gpio_reg_t * ptr_gpio)
{
    ASSERT(GPIO_ALL_PORT_CHECK(ptr_gpio));

    if(ptr_gpio == GPIOA)
    {
        __RCU_APB2_PERI_RST_ENABLE(RCU_RST_GPIOA);
        __RCU_APB2_PERI_RST_DISABLE(RCU_RST_GPIOA);
    }
    else if(ptr_gpio == GPIOB)
    {
        __RCU_APB2_PERI_RST_ENABLE(RCU_RST_GPIOB);
        __RCU_APB2_PERI_RST_DISABLE(RCU_RST_GPIOB);
    }
    else if(ptr_gpio == GPIOC)
    {
        __RCU_APB2_PERI_RST_ENABLE(RCU_RST_GPIOC);
        __RCU_APB2_PERI_RST_DISABLE(RCU_RST_GPIOC);
    }
    else if(ptr_gpio == GPIOD)
    {
        __RCU_APB2_PERI_RST_ENABLE(RCU_RST_GPIOD);
        __RCU_APB2_PERI_RST_DISABLE(RCU_RST_GPIOD);
    }
    else if(ptr_gpio == GPIOE)
    {
        __RCU_APB2_PERI_RST_ENABLE(RCU_RST_GPIOE);
        __RCU_APB2_PERI_RST_DISABLE(RCU_RST_GPIOE);
    }
}

/**@brief       Rest AFIO registers (remap, event control and EXTI configuration) to default value.
 *
 * @param[in]   None.
 *
 * @return      None.
 */
void gpio_af_deinit(void)
{
    __RCU_APB2_PERI_RST_ENABLE(RCU_RST_AFIO);
    __RCU_APB2_PERI_RST_DISABLE(RCU_RST_AFIO);
}

/**@brief       Locks GPIO pins configuration registers, The locked registers are GPIOx_CFGL,
 *              GPIOx_CFGH.The sequence is: w1 w0 w1 r0 r1
 *
 * @param[in]   ptr_gpio: pointer to GPIOx, where x can be A to E .
 *
 * @param[in]   pin: specifies the port pin to be set (Support any combination of GPIO_PIN_0 to GPIO_PIN_15).
 *
 * @return      None.
 */
void gpio_pin_lock_set(gpio_reg_t * ptr_gpio, uint16_t pin)
{
    __IO uint32_t temp = 0x00010000;
    
    ASSERT(GPIO_ALL_PORT_CHECK(ptr_gpio));
    ASSERT(GPIO_PIN_CHECK(pin));

    temp |= pin;
    
    /* write 1 */
    ptr_gpio->LOCK = temp;
    /* write 0 */
    ptr_gpio->LOCK = pin;
    /* write 1 */
    ptr_gpio->LOCK = temp;
    /* read 0 */
    temp = ptr_gpio->LOCK;
    /* read 0 */
    temp = ptr_gpio->LOCK;
}

/**@brief       Config GPIO event port and pin.
 *
 * @param[in]   event_port: specifies GPIO. It can be one of the following values: @ref GPIO_event_port.
 *
 * @param[in]   event_pin: specifies pin. It can be one of the following values: @ref GPIO_event_pin.
 *
 * @return      None.
 */
void gpio_event_config(uint8_t event_port, uint8_t event_pin)
{
    ASSERT(GPIO_ENENT_PORT_CHECK(event_port));
    ASSERT(GPIO_EVENT_PIN_CHECK(event_pin));

    MODIFY_REG(AFIO->ECR, AFIO_ECR_EOPS | AFIO_ECR_EOPIN, (event_port << 4) | event_pin);
}

/**@brief       Config GPIO remap pin.
 *
 * @param[in]   remap_peri: specifies peripherals. It can be one of the following values: @ref GPIO_remap_define.
 *
 * @param[in]   flag: ENABLE or DISABLE.
 *
 * @return      None.
 */
void gpio_pin_remap_config(uint32_t remap_peri, enable_state_t flag)
{
    ASSERT(GPIO_REMAP_CHECK(remap_peri));
    ASSERT(FUNCTIONAL_STATE_CHECK(flag));

    if(flag != DISABLE)
    {
        AFIO->MP |= remap_peri;
    }
    else
    {
        AFIO->MP &= ~remap_peri;
    }
}

/**@brief       Config GPIO exti port and pin.
 *
 * @param[in]   exti_port: specifies GPIO. It can be one of the following values: @ref GPIO_event_port.
 *
 * @param[in]   exti_pin: specifies pin. It can be one of the following values: @ref GPIO_event_pin.
 *
 * @return      None.
 */
void gpio_exti_pin_config(uint8_t exti_port, uint8_t exti_pin)
{
    uint32_t reg_tmp = 0x0;
    
    ASSERT(GPIO_EXTI_PORT_CHECK(exti_port));
    ASSERT(GPIO_EXTI_PIN_CHECK(exti_pin));

    /* Get mask */
    reg_tmp = ((uint32_t)0x0F) << ((exti_pin & 0x03) * 0x04);
    /* Clear bits */
    AFIO->EXTCFG[exti_pin >> 0x02] &= ~reg_tmp;
    /* Set value */
    AFIO->EXTCFG[exti_pin >> 0x02] |= ((uint32_t)exti_port) << ((exti_pin & (uint8_t)0x03) * 0x04);
}


/*** (C) COPYRIGHT 2021 Software Development. ***/

