/***************************************************************************//**
 * @file        cs32f10x_misc.c
 * @version     V2.0.1
 * @author      Software Development
 * @brief       This file provides all the basic misc firmware functions.
 *              
 * @copyright   Copyright (C) Software Development. All rights reserved.
 ****************************************************************************/

/*****************************************************************************
 * @includes 
*****************************************************************************/
#include "cs32f10x_misc.h"


/*****************************************************************************
 * @definitions 
*****************************************************************************/
#define AIRCR_VECTKEY_MASK    ((uint32_t)0x05FA0000)


/*****************************************************************************
 * @functions
*****************************************************************************/

/**@brief       Configures the priority grouping.
 *
 * @param[in]   priority_group: It can be one of the following values: @ref Preemption_Priority_Group
 *
 * @return      None.
 */
void nvic_priority_group_config(uint32_t priority_group)
{
  /* Check the parameters */
  ASSERT(NVIC_PRIORITY_GROUP_CHECK(priority_group));
  
  /* Set the PRIGROUP[10:8] bits according to NVIC_PriorityGroup value */
  SCB->AIRCR = AIRCR_VECTKEY_MASK | priority_group;
}

/**@brief       Initializes the NVIC peripheral according to the specified parameters in the nvic_struct.
 *
 * @param[in]   nvic_struct: pointer to a nvic_init_t structure.
 *
 * @return      None.
 */
void nvic_init(nvic_init_t* nvic_struct)
{
  uint32_t tmppriority = 0x00, tmppre = 0x00, tmpsub = 0x0F;
  
  /* Check the parameters */
  ASSERT(FUNCTIONAL_STATE_CHECK(nvic_struct->nvic_irq_enable));
  ASSERT(NVIC_PREEMPTION_PRIORITY_CHECK(nvic_struct->nvic_irq_pre_priority));  
  ASSERT(NVIC_SUB_PRIORITY_CHECK(nvic_struct->nvic_irq_sub_priority));
    
  if (nvic_struct->nvic_irq_enable != DISABLE)
  {
    /* Compute the Corresponding IRQ Priority --------------------------------*/    
    tmppriority = (0x700 - ((SCB->AIRCR) & (uint32_t)0x700))>> 0x08;
    tmppre = (0x4 - tmppriority);
    tmpsub = tmpsub >> tmppriority;

    tmppriority = (uint32_t)nvic_struct->nvic_irq_pre_priority << tmppre;
    tmppriority |=  nvic_struct->nvic_irq_sub_priority & tmpsub;
    tmppriority = tmppriority << 0x04;
        
    NVIC->IP[nvic_struct->nvic_irqchannel] = tmppriority;
    
    /* Enable the Selected IRQ Channels --------------------------------------*/
    NVIC->ISER[nvic_struct->nvic_irqchannel >> 0x05] =
      (uint32_t)0x01 << (nvic_struct->nvic_irqchannel & (uint8_t)0x1F);
  }
  else
  {
    /* Disable the Selected IRQ Channels -------------------------------------*/
    NVIC->ICER[nvic_struct->nvic_irqchannel >> 0x05] =
      (uint32_t)0x01 << (nvic_struct->nvic_irqchannel & (uint8_t)0x1F);
  }
}

/**@brief       Sets the vector table location and Offset.
 *
 * @param[in]   vec_tab: vector table is in RAM or FLASH memory.This parameter can be one of the following values:
 *              @ref NVIC_vector_mapping
 *
 * @param[in]   offset: Vector Table base offset field. This value must be a multiple of 0x200.
 *
 * @return      None.
 */
void nvic_vector_table_set(uint32_t vec_tab, uint32_t offset)
{ 
  /* Check the parameters */
  ASSERT(NVIC_VECT_MAPPING_CHECK(vec_tab));
  ASSERT(NVIC_OFFSET_CHECK(offset));  
   
  SCB->VTOR = vec_tab | (offset & (uint32_t)0x1FFFFF80);
}

/**@brief       Selects the condition for the system to enter low power mode.
 *
 * @param[in]   low_power_mode: Specifies the new mode for the system to enter low power mode.
 *              This parameter can be one of the following values: @ref NVIC_system_low_power
 *
 * @param[in]   status: ENABLE or DISABLE the mode.
 *
 * @return      None.
 */
void nvic_sys_lowpwr_config(uint8_t low_power_mode, enable_state_t status)
{
  /* Check the parameters */
  ASSERT(NVIC_LP_CHECK(low_power_mode));
  ASSERT(FUNCTIONAL_STATE_CHECK(status));  
  
  if (status != DISABLE)
  {
    SCB->SCR |= low_power_mode;
  }
  else
  {
    SCB->SCR &= (uint32_t)(~(uint32_t)low_power_mode);
  }
}

/**@brief       Configures the SysTick clock source.
 *
 * @param[in]   systick_clk_source: specifies the SysTick clock source.
 *              This parameter can be one of the following values: @ref SysTick_clock_source
 *
 * @return      None.
 */
void systick_clk_src_config(uint32_t systick_clk_source)
{
  /* Check the parameters */
  ASSERT(SYSTICK_CLK_SOURCE_CHECK(systick_clk_source));
  if (systick_clk_source == SysTick_CLKSource_HCLK)
  {
    SysTick->CTRL |= SysTick_CLKSource_HCLK;
  }
  else
  {
    SysTick->CTRL &= SysTick_CLKSource_HCLK_Div8;
  }
}


/*** (C) COPYRIGHT 2021 Software Development . ***/
