/***************************************************************************//**
 * @file        cs32f10x_rcu.c
 * @version     V2.0.1
 * @author      Software Development
 * @brief       This file provides all the basic RCC firmware functions.
 *              
 * @copyright   Copyright (C) Software Development. All rights reserved.
 ****************************************************************************/

/*****************************************************************************
 * @includes 
*****************************************************************************/
#include "cs32f10x_rcu.h"


/*****************************************************************************
 * @definitions 
*****************************************************************************/


/*****************************************************************************
 * @variables
*****************************************************************************/
static __I uint8_t APBAHB_pdiv_table[16] = {0, 0, 0, 0, 1, 2, 3, 4, 1, 2, 3, 4, 6, 7, 8, 9};
static __I uint8_t ADC_pdiv_table[4] = {2, 4, 6, 8};



/*****************************************************************************
 * @functions
*****************************************************************************/


/**@brief       Resets the RCU clock configuration to the default reset state.
 *              The default reset state including HRC ON and used as system clock source HRC, HXT and PLL OFF AHB, APB 
 *              prescaler is set to 1, CLKOUT is off.  All interrupts is disabled, but it doesn't modify the
 *              configuration of the peripheral clocks LRC, LXT and RTC clocks.
 *
 * @param       None.
 *
 * @return      None.
 */
void rcu_def_init(void)
{
    /* Enable HRC */
    RCU->CTR |= (uint32_t)0x00000001;

    /* Reset SW, HPRE, PPRE1, PPRE2, ADCPRE and MCO bits */
    RCU->CFG &= (uint32_t)0xF8FF0000;

    /* Reset HSEON, CSSON and PLLON bits */
    RCU->CTR &= (uint32_t)0xFEF6FFFF;

    /* Reset HSEBYP bit */
    RCU->CTR &= (uint32_t)0xFFFBFFFF;

    /* Reset PLLSRC, PLLXTPRE, PLLMUL and USBPRE/OTGFSPRE bits */
    RCU->CFG &= (uint32_t)0xFF80FFFF;

    /* Disable all interrupts and clear pending bits  */
    RCU->INTR = 0x009F0000;
}

/**@brief       Waits for HXT stabilization. It waits on HSERDY flag to be set and return SUCCESS. The HXT is stopped by  
 *              hardware when entering STOP and STANDBY modes.
 *
 * @param       None.
 *
 * @return      SUCCESS(oscillator is stable) or ERROR(oscillator not yet stable).
 */
error_status_t rcu_hxt_stabilization_wait(void)
{
    __IO uint32_t counter = 0;
    error_status_t flag = ERROR;
    uint32_t status = 0;
  
    /* Wait till HXT is stable and if timeout is reached exit */
    do
    {
        status = RCU->CTR & RCU_CTR_HXTSTAB;
        counter++;  
    }while((counter != HXT_STARTUP_TIMEOUT) && (status == 0));
  
    if(RCU->CTR & RCU_CTR_HXTSTAB)
    {
        flag = SUCCESS;
    }
    else
    {
        flag = ERROR;
    } 
    
    return flag;
}

/** @brief      Set external oscillator(HXT) status.
 *
 * @param[in]   HXT status,it can be one of the following values: @ref RCU_hxt_state.
 *
 * @return      None.
 */
void rcu_hxt_status_set(uint32_t rcu_hxt)
{
    ASSERT(RCU_HXT_STAT_CHECK(rcu_hxt));

    /* Reset HSEON bit */
    RCU->CTR &= ~RCU_CTR_HXTEN;
    /* Reset HSEBYP bit */
    RCU->CTR &= ~RCU_CTR_HXTBPS;

    if(rcu_hxt == RCU_HXT_ENABLE)
    {
        MODIFY_REG(RCU->CTR, RCU_CTR_HXTEN, rcu_hxt);
    }
    else if(rcu_hxt == RCU_HXT_BYPASS)
    {
        MODIFY_REG(RCU->CTR, RCU_CTR_HXTEN | RCU_CTR_HXTBPS, rcu_hxt | RCU_CTR_HXTEN);
    }
}

/**@brief       Adjusts the internal high speed RC (HRC) calibration value.
 *
 * @param[in]   value: specifies the HRC calibration trimming value (between 0 and 0x1F).
 *
 * @return      None.
 */
void rcu_hrc_calibration_adjust(uint8_t value)
{
    ASSERT(RCU_HRC_CALI_VALUE_CHECK(value));
    
    MODIFY_REG(RCU->CTR, RCU_CTR_HRCTRIM, (uint32_t)value << 3);
}

/**@brief       Configures the PLL clock source and multiplition factor.It must be used when the PLL is disabled.
 *
 * @param[in]   source: specifies the PLL selected clock source,it can be one of the following values: @ref RCU_pll_src.
 *
 * @param[in]   multiple: specifies the PLL multiplication factor,it can be one of the following values: @ref RCU_pll_multiple.
 *
 * @return      None.
 */
void rcu_pll_config(uint32_t source, uint32_t multiple)
{
    ASSERT(RCU_PLL_SOURCE_CHECK(source));
    ASSERT(RCU_PLL_MULTI_CHECK(multiple));

    MODIFY_REG(RCU->CFG, RCU_CFG_PLLSEL | RCU_CFG_PLLHXTPDIV | RCU_CFG_PLLMUF, source | multiple);
}

/**@brief       Selects the clock source to output on CLKOUT pin.
 *
 * @param[in]   source: specifies the clock source to output, it can be one of the following values: @ref RCU_clk_out.
 *
 * @return      None.
 */
void rcu_clkout_config(uint32_t source)
{
    ASSERT(RCU_CLKOUT_SRC_CHECK(source));

    MODIFY_REG(RCU->CFG, RCU_CFG_CKOSEL, source);
}

/**@brief       Configures the system clock (SYSCLK).
 *
 * @param[in]   source: specifies the clock source used as system clock. It can be one of the following values: @ref RCU_sysclk_source
 *
 * @return      None.
 */
void rcu_sysclk_config(uint32_t source)
{
    ASSERT(RCU_SYSCLK_SEL_CHECK(source));

    MODIFY_REG(RCU->CFG, RCU_CFG_SYSSW, source);
}

/**@brief       Get the system clock (SYSCLK) source.
 *
 * @param[in]   None. 
 *
 * @return      sysclk source,It can be one of the following values: @ref RCU_sysclk_source
 */
uint32_t rcu_sysclk_src_get(void)
{
    return ((RCU->CFG & RCU_CFG_SYSSS) >> 2);
}

/**@brief       Configures the APB1 clock.
 *
 * @param[in]   source: HCLK divided used as APB1 clock. It can be one of the following values: @ref RCU_pclk_source
 *
 * @return      None.
 */
void rcu_pclk1_config(uint32_t source)
{
    ASSERT(RCU_PCLK_SEL_CHECK(source));

    MODIFY_REG(RCU->CFG, RCU_CFG_PCLKPDIV, source);
}

/**@brief       Configures the APB2 clock.
 *
 * @param[in]   source: HCLK divided used as APB2 clock. It can be one of the following values: @ref RCU_pclk_source
 *
 * @return      None.
 */
void rcu_pclk2_config(uint32_t source)
{
    ASSERT(RCU_PCLK_SEL_CHECK(source));

    MODIFY_REG(RCU->CFG, RCU_CFG_HPCLKPDIV, source << 3);
}

/**@brief       Configures the AHB clock.
 *
 * @param[in]   source: SYSCLK divided used as AHB clock. It can be one of the following values: @ref RCU_hclk_source
 *
 * @return      None.
 */
void rcu_hclk_config(uint32_t source)
{
    ASSERT(RCU_HCLK_SEL_CHECK(source));

    MODIFY_REG(RCU->CFG, RCU_CFG_HCLKPDIV, source);
}

/**@brief       Configures the ADC clock.
 *
 * @param[in]   source: APB2 clock divided used as adc clock. It can be one of the following values: @ref RCU_adcclk_source
 *
 * @return      None.
 */
void rcu_adcclk_config(uint32_t source)
{
    ASSERT(RCU_ADCCLK_SEL_CHECK(source));

    MODIFY_REG(RCU->CFG, RCU_CFG_ADCPDIV, source);
}

/**@brief       Configures the USB clock.
 *
 * @param[in]   source: PLL clock divided used as USB clock. It can be one of the following values: @ref RCU_usbclk_source
 *
 * @return      None.
 */
void rcu_usbclk_config(uint32_t source)
{
    ASSERT(RCU_USBCLK_SEL_CHECK(source));

    MODIFY_REG(RCU->CFG, RCU_CFG_USBPDIV, source);
}

/**@brief       Configures the RTC clock.
 *
 * @param[in]   source: It can be one of the following values: @ref RCU_rtcclk_source
 *
 * @return      None.
 */
void rcu_rtcclk_config(uint32_t source)
{
    ASSERT(RCU_RTCCLK_SEL_CHECK(source));

    MODIFY_REG(RCU->VBDC, RCU_VBDC_RTCSRC, source);
}

/**@brief       Get RCU clock stabilization and reset flag.
 *              bit[4:0]: Indicates the position of the flag bit in the register.
 *              bit[6:5]: Indicates which register. 
 *              1: RCU->CTR
 *              2: RCU->VBDC
 *              3: RCU->STS
 *
 * @param[in]   flag: specifies the flag to get. It can be one of the following values: @ref RCU_clk_ready_reset_flag
 *
 * @param[out]  status: the flag is SET or RESET
 *
 * @return      SUCCESS OR ERROR.
 */
bit_status_t rcu_clkready_reset_flag_get(uint8_t flag)
{
    uint32_t reg = 0;
    uint32_t tmp = 0;

    ASSERT(RCU_FLAG_CLK_READY_RST_CHECK(flag));

    /* Get register index */
    tmp = flag >> 5;

    if(tmp == 1)
    {
        reg = RCU->CTR;
    }
    else if(tmp == 2)
    {
        reg = RCU->VBDC;
    }
    else if(tmp == 3)
    {
        reg = RCU->STS;
    }

    /* Get bit index */
    tmp = flag & 0x1F;

    if((reg & ((uint32_t)1 << tmp)) != RESET)
    {
        return SET;
    }
    else
    {
        return RESET;
    }
}

/**@brief       Returns the frequencies of the System, AHB and APB busses clocks.
 *
 * @param[in]   ptr_clock: pointer to a rcu_clock_t structure which will hold the clocks frequencies.
 *
 * @return      None.
 */
void rcu_clk_freq_get(rcu_clock_t *ptr_clock)
{
    uint32_t pll_mult = 0;
    uint32_t pll_src = 0;
    uint32_t temp = 0;
    uint32_t pdiv = 0;

    switch((RCU->CFG & RCU_CFG_SYSSS) >> 2)
    {
        case 0x00:      /* HRC used as system clock */
        {
            ptr_clock->sysclk_freq = HRC_VALUE;
            break;
        }
        case 0x01:      /* HXT used as system clock */
        {
            ptr_clock->sysclk_freq = HXT_VALUE;
            break;
        }
        case 0x02:      /* PLL used as system clock */
        {
            pll_src = RCU->CFG & (RCU_CFG_PLLSEL | RCU_CFG_PLLHXTPDIV);
            pll_mult = ((RCU->CFG & RCU_CFG_PLLMUF) >> 18) + 2;

            /* HRC */
            if(pll_src == RCU_PLL_SOURCE_HRC_DIV2)
            {
                ptr_clock->sysclk_freq = (HRC_VALUE >> 1) * pll_mult;
            }
            else if(pll_src == RCU_PLL_SOURCE_HXT_DIV1)
            {
                ptr_clock->sysclk_freq = HXT_VALUE * pll_mult;
            }
            else if(pll_src == RCU_PLL_SOURCE_HXT_DIV2)
            {
                ptr_clock->sysclk_freq = (HXT_VALUE >> 1) * pll_mult;
            }
            
            break;
        }
        default:
        {
            ptr_clock->sysclk_freq = HRC_VALUE;
            break;
        }
    }

    /* HCLK prescaler get */
    temp = RCU->CFG & RCU_CFG_HCLKPDIV;
    temp = temp >> 4;
    pdiv = APBAHB_pdiv_table[temp];
    /* Get HCLK frequency */
    ptr_clock->hclk_freq = ptr_clock->sysclk_freq >> pdiv;

    /* APB1 clock prescaler get */
    temp = RCU->CFG & RCU_CFG_PCLKPDIV;
    temp = temp >> 8;
    pdiv = APBAHB_pdiv_table[temp];
    /* Get APB1CLK frequency */
    ptr_clock->pclk1_freq = ptr_clock->hclk_freq >> pdiv;

    /* APB2 clock prescaler get */
    temp = RCU->CFG & RCU_CFG_HPCLKPDIV;
    temp = temp >> 11;
    pdiv = APBAHB_pdiv_table[temp];
    /* Get APB2CLK frequency */
    ptr_clock->pclk2_freq = ptr_clock->hclk_freq >> pdiv;

    /* ADC clock prescaler get */
    temp = RCU->CFG & RCU_CFG_ADCPDIV;
    temp = temp >> 14;
    pdiv = ADC_pdiv_table[temp];
    /* Get ADC clock frequency */
    ptr_clock->adc_clk_freq = ptr_clock->pclk2_freq / pdiv;
}

/*** (C) COPYRIGHT 2021 Software Development . ***/


