

#include "flash_interface.h"


#define FLASH_ERR_MASK      (uint32_t)(FMC_STS_WPERR | FMC_STS_PGERR)


static uint8_t write_counter;
static uint8_t flash_status;


uint8_t flash_operation_wait(uint32_t timeout)
{
    while(timeout--)
    {
        if(!(FLASH->STS & FMC_STS_BUSY))
        {
            if(FLASH->STS & FMC_STS_WPERR)
            {
                FLASH->STS |= FMC_STS_WPERR;
                return CMD_EXECUTE_NOPERMISSION;
            }
            if(FLASH->STS & FMC_STS_PGERR)
            {
                FLASH->STS |= FMC_STS_PGERR;
                return CMD_EXECUTE_FAIL;
            }
            return CMD_EXECUTE_SUCCESS;
        }
    }
    return CMD_EXECUTE_FAIL;
}


/**@brief       Reads data at the specified flash address into the buffer.
 * 
 * @param[in]   addr: Specifies the start address of flash.
 * 
 * @param[in]   length: Specifies the length of data by byte to be read.
 * 
 * @param[out]  pdata: Points to the buffer storing flash data.
 * 
 * @return      The status of read process.
 */
uint8_t flash_read(uint32_t addr, uint8_t length, uint8_t *pdata)
{
    uint8_t index;
    
    if(FLASH->OBSTS & FMC_OBSTS_RPROT)
    {
        return CMD_EXECUTE_NOPERMISSION;
    }
    
    for(index = 0; index < length; index++)
    {
        *(pdata + index) = *((uint8_t *)addr + index);
    }
    
    return CMD_EXECUTE_SUCCESS;
}


/**@brief       Writes data from the buffer to the specified flash address.
 * 
 * @param[in]   addr: Specifies the start address of flash.
 * 
 * @param[in]   length: Specifies the length of dat by half-word to be write.
 * 
 * @param[in]   pdata: Points to the buffer storing source data in half-word.
 * 
 * @return      The status of write process.
 */
uint8_t flash_write(uint32_t addr, uint8_t length, uint16_t *pdata)
{
    flash_status = flash_operation_wait(PROGRAME_TIMEOUT);
    
    if(flash_status == CMD_EXECUTE_SUCCESS)
    {
        FLASH->CCODE = KEY1;
        FLASH->CCODE = KEY2;
        FLASH->CTR |= FMC_CTR_PG;

        for(write_counter = 0; write_counter < length; write_counter++)
        {
            *(__IO uint16_t *)addr = *(pdata + write_counter);
            flash_status = flash_operation_wait(PROGRAME_TIMEOUT);
            if(flash_status != CMD_EXECUTE_SUCCESS)
            {
                break;
            }
            addr += 2;
        }
        
        FLASH->CTR &= ~(uint32_t)FMC_CTR_PG;
        FLASH->CTR |= FMC_CTR_LOCK;
    }
    return flash_status;
}


/**@brief       Erases the page of the specified address.
 * 
 * @param[in]   addr: Specifies the address of flash.
 * 
 * @return      The status of page erase process.
 */
uint8_t flash_page_erase(uint32_t addr)
{
    flash_status = flash_operation_wait(ERASE_TIMEOUT);
    
    if(flash_status == CMD_EXECUTE_SUCCESS)
    {
        FLASH->CCODE = KEY1;
        FLASH->CCODE = KEY2;
        FLASH->CTR |= FMC_CTR_PERS;
        FLASH->ERSADR = addr;
        FLASH->CTR |= FMC_CTR_START;
        
        flash_status = flash_operation_wait(ERASE_TIMEOUT);
        
        FLASH->CTR &= ~(uint32_t)FMC_CTR_PERS;
        FLASH->CTR |= FMC_CTR_LOCK;
    }
    return flash_status;
}


/**@brief       Erases the page of the specified address.
 * 
 * @param[in]   addr: Specifies the address of flash.
 * 
 * @return      The status of chip erase process.
 */
uint8_t flash_chip_erase(void)
{
    flash_status = flash_operation_wait(ERASE_TIMEOUT);
    
    if(flash_status == CMD_EXECUTE_SUCCESS)
    {
        FLASH->CCODE = KEY1;
        FLASH->CCODE = KEY2;
        FLASH->CTR |= FMC_CTR_CHIPERS;
        FLASH->CTR |= FMC_CTR_START;
        
        flash_status = flash_operation_wait(ERASE_TIMEOUT);
        
        FLASH->CTR &= ~(uint32_t)FMC_CTR_CHIPERS;
        FLASH->CTR |= FMC_CTR_LOCK;
    }
    return flash_status;
}
